# -*- coding: utf-8 -*-

from Permanencia import permanencia
import sys

from Modelo import modelo
from Vista import vista

class ControladorInicio:
    """
    Controlador inicial. Implementa los casos de uso CU04 - Elimina documentos y CU05 - Visualiza documentos.

    Controla la vista Main, y permite acceder al resto de controladores.

    Atributos:
        - vista (vista.Main)
        - dao (permanencia.DAO)
        - documentos (list: modelo.Documento)
    """
    def __init__(self):
        self.dao = permanencia.DAO()
        self.documentos = None
        self.vista = vista.Main(self)
        self.vista.mainloop()

    def cargaDocumentosSistema(self,resultado_queue,progreso_queue):
        """
        Carga los documentos almacenados en la base de datos al atributo `documentos` y los almacena en la queue compartida con la vista.

        Argumentos:
            - resultado_queue (queue.Queue): Cola compartida con la vista en la que se deben almacenar los documentos cargados.
            - progreso_queue (queue.LifoQueue): Cola compartida con la vista en la que se debe almacenar 'FIN' cuando se haya terminado la lectura.
        """
        self.dao.conectar()
        self.documentos = self.dao.getDocumentos()
        self.dao.close()
        for f in self.documentos:
            resultado_queue.put(f)
        progreso_queue.put("FIN")


    def anyadeDocumentos(self,documentos):
        """
        Actualiza la lista de documentos en atributos con los documentos indicados.

        Argumentos:
            - documentos (list:modelo.Documento)
        """
        self.documentos.extend(documentos)

    def cargaContenidoDocumento(self,documento):
        """
        Accede al contenido del documento seleccionado y ordena a la vista que
        lo cargue.

        Argumentos:
            - documento (modelo.Documento): Instancia del documento
        """
        idF = documento.getId()
        #lo buscamos en nuestra lista de documentos
        documento = [f for f in self.documentos if f.getId()==idF][0]
        self.vista.muestraContenidoDocumento(documento)

    def eliminaDocumentos(self,documentos,progreso_queue):
        """
        Elimina del sistema los documentos indicados.
        Argumentos:
            - documentos (list:modelo.Documento)
            - progreso_queue (queue.LifoQueue()): Queue compartida con la vista donde el controlador debe introducir 'FIN' cuando haya terminado de eliminar los documentos.
        """
        self.dao.conectar()
        self.dao.deleteDocumentos(documentos)
        self.dao.close()

        #los eliminamos de la lista de documentos
        idsAEliminar = [f.getId() for f in documentos]
        self.documentos = [f for f in self.documentos if f.getId() not in idsAEliminar]

        progreso_queue.put('FIN')


    def importaFicheros(self):
        """
        Crea una instancia de ControladorImporta y la ejecuta.
        """
        ControladorImporta(self).run()

    def experimentoClasificacion(self):
        """
        Crea una instancia de ControladorClasifica, pasandole los
        documentos en el sistema, y la ejecuta.
        """
        ControladorClasifica(self).run()

    def experimentoEntrenamiento(self):
        """
        Crea una instancia de ControladorEntrena, pasandole los
        documentos en el sistema, y la ejecuta.
        """
        ControladorEntrena(self).run()

    def salir(self):
        """
        Cierra la aplicación.
        """
        self.vista.quit()
        sys.exit(0)


from Procesado_textos.sgml_parser import SGMLParser

class ControladorImporta:
    """
    Implementa los casos de uso CU01 - Importa ficheros estructurados, CU02 - Importa ficheros de texto plano, CU03 - Edita atributos documentos y CU05 - Visualiza documentos.

    Controla la vista Importa.

    Atributos:
        - vista (vista.Importa)
        - dao (permanencia.DAO)
        - controladorPadre (controlador.ControladorInicio)

    Argumentos:
        - controladorPadre (controlador.ControladorInicio): Instancia del controlador inicial de la aplicación.
    """
    def __init__(self,controladorPadre):
        self.dao = permanencia.DAO()
        self.vista = vista.Importa(self,controladorPadre.vista)
        self.controladorPadre = controladorPadre

    def run(self):
        """
        Manda a la vista cargar la pantalla en la que pide la apertura de ficheros.
        """
        self.vista.pideFicheros()

    def parseaFicheros(self,ficheros,thread_queue,progreso_queue):
        """
        Parsea la lista de rutas a ficheros. El sistema soporta los ficheros .sgml con estructura:

        <DOC>
            <TITLE></TITLE>
            <CATEGORY></CATEGORY>
            <TEXT></TEXT>
        </DOC>

        y generará un modelo.Documento por cada DOC, con título definido por TITLE, clasificación por CATEGORY, y cuerpo por TEXT.

        Si no existen las etiquetas <DOC> o <TEXT> se introducirá un token 'ERROR path' en la queue de progreso, con el `path` del fichero que ha generado el error y se detendrá el parseo.

        Los ficheros con cualquier otra extensión leen su contenido al cuerpo de un modelo.Documento.

        Argumentos:
            - ficheros (list: str): Lista de rutas, absolutas o relativas, a los ficheros a parsear.
            - thread_queue (queue.Queue): Queue compartida con la vista donde el controlador debe introducir los resultados.
            - progreso_queue (queue.LifoQueue): Queue compartida con la vista donde el controlador debe introducir el número de ficheros de la lista que ha parseado.
        """
        for i,path in enumerate(ficheros):
            fa = modelo.Documento()

            if(path[-5:]==".sgml"):
                try:
                    list_fa = SGMLParser.parse(path)
                except AttributeError:
                    progreso_queue.put('ERRORSGML '+str(path))
                    return

                for f in list_fa:
                    thread_queue.put(f)
                progreso_queue.put(i)

            else:
                try:
                    file = open(path,mode='r')
                    fa.setCuerpo(file.read().replace("'",'"'))
                    fa.setPath(path)
                    thread_queue.put(fa)
                    progreso_queue.put(i)
                except:
                    progreso_queue.put('ERRORPLANO '+str(path))
                    return


    def guardarCambios(self,documentos,progreso_queue,thread_queue):
        """
        Se almacenan en la permanencia sistema los documentos indicados.

        Al finalizar manda un mensaje al controlador padre para añadir los documentos a su lista.

        Argumentos:
                - documentos (list: modelo.Documento)
                - progreso_queue (queue.Queue): Queue compartida con la vista donde el controlador debe introducir el token 'FINAL' cuando se termine el guardado.
                - thread_queue (queue.LifoQueue): Queue compartida con la vista donde el controlador debe introducir los resultados.
        """
        self.dao.conectar()
        nuevos_documentos = self.dao.uploadDocumentos(documentos)
        self.dao.close()
        progreso_queue.put('FINAL')
        for f in nuevos_documentos:
            thread_queue.put(f)
        self.controladorPadre.anyadeDocumentos(nuevos_documentos)


from Procesado_textos.conversor_textos import ConversorTextos
from Procesado_textos.normalizador import Normalizador
from Redes.redes import Elman, Jordan

from numpy.random import seed
from numpy import zeros, array, trace
from sklearn.model_selection import StratifiedShuffleSplit, StratifiedKFold

from collections import Counter

class ControladorEntrena:
    """
    Implementa los casos de uso CU06 - Entrena red, CU07 - Guarda modelo y CU08 - Entrena más epocas.

    Atributos:
        - vista (vista.Entrena)
        - dao (permanencia.DAO)
        - controladorPadre (controlador.ControladorInicio)
        - documentos (list: modelo.Documento): documentos almacenados en el sistema que pueden ser utilizados para el entrenamiento. Esto es, solo aquellos que tengan clasificación almacenada.
        - experimento (modelo.ExperimentoEntrenamiento): almacena los datos necesarios para realizar el experimento.

    Argumentos:
        - controladorPadre (controlador.ControladorInicio): Instancia del controlador inicial de la aplicación.
        - documentos (list: modelo.Documento): Lista de documentos en el sistema.
    """
    def __init__(self,controladorPadre):
        self.vista = vista.Entrena(self)
        self.dao = permanencia.DAO()
        self.controladorPadre = controladorPadre
        #sacamos solo los que tengan clasificación
        self.documentos = [f for f in controladorPadre.documentos if f.getClasificacion() != ""]
        self.experimento = None

    def run(self):
        """
        Selecciona los documentos con clasificación, y manda a la vista cargar la pantalla en la que pide la selección de documentos y parámetros del modelo, e inicia su instancia de vista, o manda cargar un mensaje de error si no existen documentos en el sistema.
        """
        if(len(self.documentos)==0):
            self.vista.muestraErrorSinDocumentos()
            self.vista.destroy()
            return
        else:
            self.vista.cargaPantallaSeleccion(self.documentos)

    def runExperimento(self,experimento,thread_queue,progreso_queue):
        """
        Carga los documentos, la red, y sus parámetros acorde con el experimento recibido,
        manda a la vista que cargue la pantalla de progreso del entrenamiento, y
        ejecuta las epocas indicadas en el experimento.

        Durante el entrenamiento, introduce en las queues compartida con la vista el progreso del experimento.

        Argumentos:
            - experimento (modelo.ExperimentoEntrenamiento)
            - thread_queue (queue.LifoQueue): Queue compartida con la vista donde el controlador debe ir introduciendo los resultados del experimento cada epoca.
            - progreso_queue (queue.Queue): Queue compartida con la vista donde el controlador debe introducir el progreso del experimento.

        Lanza:
            - SystemError: la función lanza esta excepción si no existe la implementación para algún método o red.
        """

        self.contadorEpocas = 0
        self.queue = thread_queue
        self.progreso = progreso_queue

        ids = [fi.getId() for fi in experimento.getConjuntoEntrenamiento()]

        self.conjunto = [f for f in self.documentos if f.getId() in ids]
        self.experimento = experimento
        self.resultados = modelo.EstadoEntrenamiento()

        self.modelo = self.experimento.getModelo()

        seed(self.experimento.getSemillaAleatoria())

        self.X = [f.getCuerpo() for f in self.conjunto]
        self.Y = [f.getClasificacion() for f in self.conjunto]

        if(self.experimento.getMetodoValidacion() == 'HOLDOUT'):
            try:
                ((self.X_entrenamiento,self.Y_entrenamiento), (self.X_validacion,self.Y_validacion)) = self.__holdout(self.X,self.Y,self.experimento.getParametroValidacion(),self.experimento.getSemillaAleatoria())
            except:
                self.progreso.put('ERR HOLDOUT')
                return

        elif(self.experimento.getMetodoValidacion() == 'ENTRENAMIENTO'):
                self.X_entrenamiento = self.X
                self.Y_entrenamiento = self.Y
                self.X_validacion = self.X
                self.Y_validacion = self.Y

        if(self.experimento.getMetodoValidacion() == 'HOLDOUT' or self.experimento.getMetodoValidacion()=='ENTRENAMIENTO'):

            self.evolucion_tasa_entr = list()
            self.evolucion_tasa_val = list()

            self.__preparaDocumentos()
            self.__inicializaExperimento()

            #añadimos los resultados iniciales a la queue de resultado
            self.queue.put(self.resultados)

            self.__ejecutaEpocas()

            self.modelo = self.__getModeloActualizado()
            self.experimento.setModelo(self.modelo)

            self.queue.put(self.resultados)

            self.progreso.put("FIN")

        elif(self.experimento.getMetodoValidacion() == 'CRUZADA'):
            evolucion_tasa_entr_media = zeros((self.experimento.getNumeroEpocas()))
            evolucion_tasa_val_media = zeros((self.experimento.getNumeroEpocas()))

            mejor_experimento = self.experimento
            mejor_tasa = -1.0

            folds = self.experimento.getParametroValidacion()
            self.resultados.setExperimentosARealizar(folds)

            freq = Counter(self.Y)
            freq = freq.most_common()
            if(freq[-1][1]<folds):
                self.progreso.put('ERR CRUZADA')
                return

            generador_folds = self.__kfold(folds,self.X,self.Y,self.experimento.getSemillaAleatoria())

            self.X = array(self.X)
            self.Y = array(self.Y)
            cont_experimento = 0
            for indice_ent, indice_val in generador_folds:

                self.contadorEpocas = 0

                self.evolucion_tasa_entr = list()
                self.evolucion_tasa_val = list()

                self.X_entrenamiento = self.X[indice_ent]
                self.Y_entrenamiento = self.Y[indice_ent]

                self.X_validacion = self.X[indice_val]
                self.Y_validacion = self.Y[indice_val]


                self.__preparaDocumentos()
                self.__inicializaExperimento()

                #debemos realizar un entrenamiento por cada uno de los folds
                self.resultados.setExperimentosRealizados(cont_experimento)

                self.queue.put(self.resultados)


                self.__ejecutaEpocas()

                #almacenamos el mejor modelo para guardarlo al finalizar los experimentos
                #y la matriz de confusion para poder mostrarla
                if(self.resultados.getTasasValidacion()[-1]>mejor_tasa):
                    mejor_tasa = self.resultados.getTasasValidacion()[-1]
                    mejor_experimento.setModelo(self.__getModeloActualizado())

                    mejor_experimento.setConjuntoValidacion([self.X_validacion,self.Y_validacion])
                    mejor_experimento.setConjuntoEntrenamiento([self.X_entrenamiento,self.Y_entrenamiento])

                    mejor_matriz = self.resultados.getMatrizConfusion()

                evolucion_tasa_entr_media += array(self.resultados.getTasasEntrenamiento())/folds
                evolucion_tasa_val_media += array(self.resultados.getTasasValidacion())/folds


                self.progreso.put('NUEVO EXPERIMENTO')
                cont_experimento+=1

            #actualizamos los resultados a las medias de las tasas y la mejor matriz de confusion
            self.resultados.setTasasEntrenamiento(evolucion_tasa_entr_media)
            self.resultados.setTasasValidacion(evolucion_tasa_val_media)
            self.resultados.setMatrizConfusion(mejor_matriz)

            self.experimento = mejor_experimento

            self.queue.put(self.resultados)
            self.progreso.put('FIN')

        else:
            raise SystemError('No existe implementación para la validación %s' % self.experimento.getMetodoValidacion())


    def __preparaDocumentos(self):
        orden = self.experimento.getOrdenVocabulario()
        if(self.experimento.getIdioma()=='español'):
            idioma = 'spanish'
        else:
            idioma = self.experimento.getIdioma()

        self.conversorTexto = ConversorTextos(idioma=idioma,stop=True,
                                              extrastop=self.experimento.getStopWordsExtra(),
                                              orden=orden, semilla=self.experimento.getSemillaAleatoria())

        self.X_entrenamiento = self.conversorTexto.fit_transform(self.X_entrenamiento)
        self.X_validacion = self.conversorTexto.transform(self.X_validacion)

         #el conversor devuelve el token #UNK# si la palabra en alguno de los conjuntos no está
         #en el vocabulario. se elimina este token de todos los arrays
        self.X_entrenamiento = [[x for x in doc if x!='#UNK#'] for doc in self.X_entrenamiento]
        self.X_validacion = [[x for x in doc if x!='#UNK#'] for doc in self.X_validacion]

        #NORMALIZAR
        self.normalizador = Normalizador()
        self.X_entrenamiento = self.normalizador.fit_transform(self.X_entrenamiento)
        self.X_validacion = self.normalizador.transform(self.X_validacion)

    def __inicializaExperimento(self):
        #INICIALIZAMOS EXPERIMENTO
        nH = self.experimento.getModelo().getNumeroNeuronasOculta()
        fA = self.experimento.getFactorAprendizaje()
        fI = self.experimento.getFactorInercia()
        numClases = len(set(self.Y_entrenamiento))

        if(self.modelo.getTipoRed()=='ELMAN'):
            self.red = Elman(nH,numClases,1,fA,fI,self.experimento.getSemillaAleatoria())
        elif(self.modelo.getTipoRed()=='JORDAN'):
            self.red = Jordan(nH,numClases,1,fA,fI,self.experimento.getSemillaAleatoria())
        else:
            raise SystemError('No existe implementación para red tipo %s'% self.modelo.getTipoRed)

        self.red.entrena(self.X_entrenamiento,self.Y_entrenamiento,self.X_validacion,self.Y_validacion)

        proporcionesClases = Counter(self.Y_entrenamiento)
        proporcionesClases = proporcionesClases.most_common()

        clases = [p[0] for p in proporcionesClases]
        proporciones = [p[1] for p in proporcionesClases]
        proporciones = [p/sum(proporciones) for p in proporciones]

        #rellenamos los resultados del entrenamiento
        self.resultados.setEpocasARealizar(self.experimento.getNumeroEpocas())
        self.resultados.setEpocasRealizadas(self.contadorEpocas)
        self.resultados.setClases(clases)
        self.resultados.setProporcionClases(proporciones)
        self.resultados.setTamEntrenamiento(len(self.X_entrenamiento))
        self.resultados.setTamValidacion(len(self.X_validacion))


    def __holdout(self,X,Y,porc, semilla=None):
        freq = Counter(Y)
        freq = freq.most_common()
        if(freq[-1][1]<2):
            raise ValueError("Conjunto para holdout incorrecto")
        else:
            X = array(X)
            Y = array(Y)
            ss = StratifiedShuffleSplit(n_splits=1,test_size=porc,random_state=semilla)
            for train,test in ss.split(X,Y):
                X_train, X_test = X[train], X[test]
                Y_train, Y_test = Y[train], Y[test]
            return ((X_train, Y_train), (X_test, Y_test))

    def __kfold(self,k,X,Y,semilla=None):
        ss = StratifiedKFold(n_splits=k,shuffle=True,random_state=semilla)
        return ss.split(X,Y)

    def reanudaExperimento(self,numeroEpocas,thread_queue,progreso_queue):
        """
        Permite reanudar el experimento desde el estado en el que se está
        la red para un número de epocas extras.

        Para la validación cruzada, se entrena el modelo con la mejor tasa de validación.

        Tras la nueva tanda de epocas, manda los nuevos resultados a la vista.

        Argumentos:
            - numeroEpocas (int): Número de epocas extra a iterar.
            - thread_queue (queue.LifoQueue): Queue compartida con la vista donde el controlador debe ir introduciendo los resultados del experimento cada epoca.
            - progreso_queue (queue.Queue): Queue compartida con la vista donde el controlador debe introducir el progreso del experimento.
        """
        epocas_hechas = self.contadorEpocas
        self.experimento.setNumeroEpocas(numeroEpocas+epocas_hechas)

        self.queue = thread_queue
        self.progreso = progreso_queue

        #Con la validación cruzada, tenemos que cargar el modelo en experimento
        if(self.experimento.getMetodoValidacion()=='CRUZADA'):
            self.X_entrenamiento = self.experimento.getConjuntoEntrenamiento()[0]
            self.X_validacion = self.experimento.getConjuntoValidacion()[0]
            self.Y_entrenamiento = self.experimento.getConjuntoEntrenamiento()[1]
            self.Y_validacion = self.experimento.getConjuntoValidacion()[1]
            #no es necesario preparar los documentos, ya están preparados
            self.__inicializaExperimento()
            #inicializamos un nuevo experimento

        self.resultados.setEpocasARealizar(self.experimento.getNumeroEpocas())
        self.resultados.setEpocasRealizadas(epocas_hechas)

        self.queue.put(self.resultados)

        self.__ejecutaEpocas()

        self.queue.put(self.resultados)
        self.progreso.put("FIN")

    def __ejecutaEpocas(self):
        #comenzamos a iterar en la red
        while(self.contadorEpocas < self.experimento.getNumeroEpocas()):
            self.red.epoca()

            #actualizamos la vista y guardamos los resultados
            self.contadorEpocas += 1
            tasa_entr = trace(self.red.score(self.X_entrenamiento,self.Y_entrenamiento)[0])/(len(self.X_entrenamiento))
            tasa_val = trace(self.red.score(self.X_validacion,self.Y_validacion)[0])/(len(self.X_validacion))

            self.evolucion_tasa_entr.append(tasa_entr)
            self.evolucion_tasa_val.append(tasa_val)

            self.resultados.setTasasEntrenamiento(self.evolucion_tasa_entr)
            self.resultados.setTasasValidacion(self.evolucion_tasa_val)
            self.resultados.setEpocasRealizadas(self.contadorEpocas)

            self.queue.put(self.resultados)

        matrizConfusion = self.red.score(self.X_validacion,self.Y_validacion)
        self.resultados.setMatrizConfusion({'matriz':matrizConfusion[0],'filas':matrizConfusion[1],
                                            'columnas':matrizConfusion[2]})

    def guardaModelo(self,nombre):
        """
        Almacena el modelo en el atributo `experimento` en el sistema con el nombre dado.

        Argumentos:
            - nombre (str)
        """
        #actualizamos modelo

        m = self.experimento.getModelo()
        m.setNombre(nombre)

        self.dao.conectar()
        self.dao.uploadModelo(m)
        self.dao.close()

    def __getModeloActualizado(self):
        out = modelo.Modelo()
        out.setClases(self.red.getClases())
        pesos, bias = self.red.getPesos()
        out.setPesos(pesos)
        out.setBias(bias)
        out.setVocabulario(self.conversorTexto.getVocabulario())
        m = self.normalizador.getMax()
        out.setMaximoNormalizado(m)
        m = self.normalizador.getMin()
        out.setMinimoNormalizado(m)
        #algunos atributos son comunes a todos los modelos obtenidos de varios experimentos:
        out.setNumeroNeuronasOculta(self.modelo.getNumeroNeuronasOculta())
        out.setTipoRed(self.modelo.getTipoRed())
        return out


class ControladorClasifica:
    """
    Implementa los casos de uso CU09 - Clasifica documentos y CU05 - Visualiza documentos.

    Atributos
    ----------------------
    - vista (vista.Clasifica)
    - dao (permanencia.DAO)
    - controladorPadre (controlador.ControladorInicio)
    - documentos (list: modelo.Documento): documentos almacenados en el sistema que pueden ser utilizados para la clasificación.
    - experimento (modelo.ExperimentoClasificacion): almacena los datos necesarios para realizar el experimento.
    - modelos (list: modelo.Modelos): Lista de modelos en el sistema.

    Argumentos
    ---------------------
    - controladorPadre (controlador.ControladorInicio): Instancia del controlador inicial de la aplicación.
    - documentos (list: modelo.Documento): Lista de documentos en el sistema.
    """
    def __init__(self,controladorPadre):
        self.vista = vista.Clasifica(self)
        self.dao = permanencia.DAO()
        self.controladorPadre = controladorPadre
        self.documentos = self.controladorPadre.documentos

        self.dao.conectar()
        self.modelos = self.dao.getModelos()
        self.dao.close()

    def run(self):
        """
        Manda a la vista cargar la pantalla en la que pide la selección de documentos
        y modelo, e inicia su instancia de vista, o manda mostrar un mensaje de error si no existen modelos o documentos en el sistema.
        """
        if(len(self.documentos)==0):
            self.vista.muestraErrorSinDocumentos()
            self.vista.destroy()
            return
        elif(len(self.modelos)==0):
            self.vista.muestraErrorSinModelos()
            self.vista.destroy()
            return
        else:
            self.vista.cargaPantallaSeleccion(self.documentos,self.modelos)

    def runExperimento(self,experimento,progreso_queue,thread_queue):
        """
        Inicializa la red acorde con el experimento recibido, y ejecuta la
        clasificación de los documentos.

        Al terminar, manda a la vista cargar los resultados obtenidos.

        Argumentos:
            - experimento (modelo.ExperimentoClasificacion)
            - progreso_queue (queue.LifoQueue): Queue compartida con la vista donde el controlador debe introducir el string 'FIN' cuando haya finalizado.
            - thread_queue (queue.Queue): Queue compartida con la vista donde el controlador debe introducir los resultados.
        """
        ids = [fi.getId() for fi in experimento.getConjuntoClasificacion()]

        conjunto = [f for f in self.documentos if f.getId() in ids]
        modelo = [m for m in self.modelos if m.getId() == experimento.getModelo().getId()][0]

        experimento.setModelo(modelo)
        experimento.setConjuntoClasificacion(conjunto)

        self.experimento = experimento

        self.tipoRed = modelo.getTipoRed()

        self.numeroNeuronasOculta = modelo.getNumeroNeuronasOculta()

        self.clases = modelo.getClases()
        numClases = len(self.clases)

        if(modelo.getTipoRed()=='ELMAN'):
            self.red = Elman(self.numeroNeuronasOculta,numClases,1,0,0,None)
        elif(modelo.getTipoRed()=='JORDAN'):
            self.red = Jordan(self.numeroNeuronasOculta,numClases,1,0,0,None)
        else:
            raise SystemError('No existe implementación para red tipo %s'% modelo.getTipoRed)

        self.pesos = modelo.getPesos()
        self.bias = modelo.getBias()

        self.vocabulario = modelo.getVocabulario()

        self.red.setPesos(self.pesos,self.bias)
        self.red.setClases(self.clases)

        self.conversorTexto = ConversorTextos()
        self.conversorTexto.setVocabulario(self.vocabulario)

        self.X = [f.getCuerpo() for f in experimento.getConjuntoClasificacion()]

        self.X = self.conversorTexto.transform(self.X)

        #eliminamos las palabras no presentes en el vocabulario
        self.X = [[x for x in doc if x!='#UNK#'] for doc in self.X]

        self.normalizador = Normalizador()
        self.normalizador.setMin(modelo.getMinimoNormalizado())
        self.normalizador.setMax(modelo.getMaximoNormalizado())

        self.X = self.normalizador.transform(self.X)

        clasificaciones_res = self.red.clasifica(self.X)

        for i,f in enumerate(experimento.getConjuntoClasificacion()):
            f.setClasificacion(clasificaciones_res[i][0])
            thread_queue.put(f)

        progreso_queue.put('FIN')

    def cargaContenidoDocumento(self,documento):
        """
        Accede al contenido del documento seleccionado y ordena a la vista que
        lo cargue.

        Argumentos
        ---------------------
        - documento (modelo.Documento): Instancia del documento
        """
        idF = documento.getId()
        #leemos el documento de la base de datos para obtener su clasificación real
        self.dao.conectar()
        documento = self.dao.getDocumento(idF)
        self.dao.close()

        self.vista.muestraContenidoDocumento(documento)
