# -*- coding: utf-8 -*-
"""
Paquete de elementos reutilizables por la vista.

Created on Mon Mar 12 09:23:14 2018

@author: Esther Cuervo Fernández
"""

import tkinter as tk
from tkinter import ttk

class ScrollableLabelFrame(ttk.LabelFrame):
    """
    Cuadro con etiqueta de nombre que contiene una tabla que permite hacer scroll vertical.

    Argumentos:
        - master (Tkinter.Frame/Tkinter.Toplevel): Ventana o frame en el que insertar el elemento.
        - nombre (str): Nombre para la etiqueta. (por defecto "")

    Atributos:
        - scrollbar (Tkinter.Scrollbar)
        - table (Tkinter.Treeview)
    """
    def __init__(self,master,nombre=""):
        self.master = master

        ttk.LabelFrame.__init__(self,master)

        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(0,weight=1)


        self.__scrollbar = tk.Scrollbar(self)
        self.__tabla = ttk.Treeview(self,show="headings",yscrollcommand=self.__scrollbar.set,name=nombre)
        self.__tabla.grid(row=0,column=0,sticky='nsew',padx=10,pady=10)

        self.__scrollbar.config(command=self.__tabla.yview)
        self.__scrollbar.grid(row=0,column=2,sticky='wns')

    def getTree(self):
        """
        Devuelve la tabla.

        Devuelve:
            - tabla (Tkinter.ttk.Treeview)
        """
        return self.__tabla

    def getScrollbar(self):
        """
        Devuelve la barra de scroll.

        Devuelve:
            - scrollbar (Tkinter.Scrollbar)
        """
        return self.__scrollbar

    def setHeadings(self,heads):
        """
        Establece los títulos de columna de la tabla.

        Argumentos:
            - heads (list:str): títulos de columna
        """
        self.__tabla.config(columns=heads)
        for head in heads:
            self.__tabla.heading(head,text=head)

    def addDat(self,dat):
        """
        Inserta el dato en la tabla.

        `dat` debe tener la misma longitud que los títulos de columna introducidos con `setHeadings`.

        Argumentos:
            - dat (list:str)
        """
        self.__tabla.insert("",'end',values=dat)

class FramesSeleccion(tk.Frame):
    """
    Cuadro de selección con etiquetas que permite hacer scroll.

    Atributos:
        - master (Tkinter.Frame/Tkinter.Toplevel): Ventana o frame en el que insertar el elemento.
        - sel (ScrollableLabelFrame): Frame de ficheros seleccionados.
        - noSel (ScrollableLabelFrame): Framd e ficheros no seleccionados.
    Argumentos:
        - master (Tkinter.Frame/Tkinter.Toplevel): Ventana o frame en el que insertar el elemento.
    """
    def __init__(self,master):
        self.master = master
        ttk.Frame.__init__(self,master)

        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(0,weight=1)
        self.grid_columnconfigure(2,weight=1)

        self.__noSel = ScrollableLabelFrame(self)
        self.__noSel.config(text="Documentos no seleccionados")
        self.__noSel.grid(row=0,column=0,sticky='nswe')

        self.__sel = ScrollableLabelFrame(self)
        self.__sel.config(text="Documentos seleccionados")
        self.__sel.grid(row=0,column=2,sticky='nswe')

        # Botones de seleccion
        frame_seleccion = ttk.Frame(self)
        frame_seleccion.grid(row=0,column=1,padx=5)
        self.__selec_boton = ttk.Button(frame_seleccion,text="-->",command=self.__eventSel)
        self.__selec_boton.config(state='disabled')

        self.__selec_boton.grid(row=0,column=0)
        self.__deselec_boton = ttk.Button(frame_seleccion,text="<--",command=self.__eventDes)
        self.__deselec_boton.grid(row=1,column=0)
        self.__deselec_boton.config(state='disabled')

        # si hay seleccion en la tabla correspondiente activamos el botón
        self.__sel.getTree().bind('<<TreeviewSelect>>',self.__eventHaySel)
        self.__noSel.getTree().bind('<<TreeviewSelect>>',self.__eventHaySelDe)

        # doble click selecciona/deselecciona la opcion
        self.__sel.getTree().bind('<Double-ButtonRelease-1>',self.__eventDes)
        self.__noSel.getTree().bind('<Double-ButtonRelease-1>',self.__eventSel)

    def __eventSel(self,event=None):
        widgSel = self.__sel.getTree()
        widgDes = self.__noSel.getTree()

        idItemSeleccionado = widgDes.selection()

        for iid in idItemSeleccionado:
            widgSel.insert('',0,iid=iid,values=widgDes.item(iid)['values'])
            widgSel.see(iid)

            widgDes.delete(iid)
        self.__selec_boton.state(['disabled'])

    def __eventDes(self,event=None):
        widgSel = self.__sel.getTree()
        widgDes = self.__noSel.getTree()
        idItemSeleccionado = widgSel.selection()

        for iid in idItemSeleccionado:
            widgDes.insert('',0,iid=iid,values=widgSel.item(iid)['values'])
            widgDes.see(iid)

            widgSel.delete(iid)

        self.__deselec_boton.state(['disabled'])

    def __eventHaySel(self,event):
        boton = self.__deselec_boton
        self.__checkSel(event.widget,boton)

    def __eventHaySelDe(self,event):
        boton = self.__selec_boton
        self.__checkSel(event.widget,boton)

    def __checkSel(self,widg,boton):
        if(len(widg.get_children())==0):
           boton.state(['disabled'])
        else:
            boton.state(['!disabled'])

    def getSelTree(self):
        """
        Devuelve la tabla de elementos seleccionados.

        Devuelve:
            - sel (Tkinter.Treeview)
        """
        return self.__sel.getTree()

    def getNoSelTree(self):
        """
        Devuelve la tabla de elementos no seleccionados.

        Devuelve:
            - noSel (Tkinter.Treeview)
        """
        return self.__noSel.getTree()

    def setHeadings(self,data):
        """
        Establece los títulos de columna de las tablas.

        Argumentos:
            - heads (list:str): títulos de columna
        """
        self.__noSel.setHeadings(data)
        self.__sel.setHeadings(data)

    def addDat(self,data):
        """
        Inserta los datos en la tabla de no seleccionados.

        `dat` debe tener la misma longitud que los títulos de columna introducidos con `setHeadings`.

        Argumentos:
            - data (list:str)
        """
        self.__noSel.addDat(data)

    def getSelection(self):
        """
        Devuelve los elementos en la tabla de seleccionados.

        Devuelve:
            - selection (list:list:str)
        """
        selection = self.__sel.getTree().get_children()
        output = list()
        for kid in selection:
            output.append(self.__sel.getTree().item(kid)['values'])
        return output

class TextoDocumento(tk.Toplevel):
    """
        Ventana que muestra el contenido de un documento seleccionado.
    """
    def __init__(self):
        tk.Toplevel.__init__(self)

        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(2,weight=1)

        self.__titulo = ttk.Label(self)
        self.__titulo.grid(row=0,column=0,sticky='w',padx=20,pady=5)

        self.__clasificacion = ttk.Label(self)
        self.__clasificacion.grid(row=1,column=0,sticky='w',padx=20)

        frameContenido = ttk.Frame(self)
        frameContenido.grid(row=2,column=0,sticky='nsew',padx=20,pady=10)

        frameContenido.grid_columnconfigure(0,weight=1)
        frameContenido.grid_rowconfigure(0,weight=1)

        self.__contenido = tk.Text(frameContenido,wrap='word',font="Helvetica")
        self.__contenido.grid(row=0,column=0,sticky='nsew')

        scrollbar = tk.Scrollbar(frameContenido)
        scrollbar.grid(row=0,column=1,sticky='sn')

        self.__contenido.config(yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.__contenido.yview)

    def muestraContenidoDocumento(self,doc):
        """
        Muestra el contenido del documento: su título, su clasificación, y su contenido.

        Argumentos:
            - doc (modelo.Documento)
        """
        self.title("Contenido del documento "+str(doc.getId()))
        if(doc.getTitulo()!=""):
            self.__titulo.config(text="Título:\t\t"+doc.getTitulo())
        else:
            self.__titulo.config(text="SIN TÍTULO")
        if(doc.getClasificacion()!=""):
            self.__clasificacion.config(text="Clasificación:\t"+doc.getClasificacion())
        else:
            self.__clasificacion.config(text="SIN CLASIFICACIÓN")
        self.__contenido.insert(tk.END,doc.getCuerpo())
        self.__contenido.config(state='disabled')


class PantallaProgreso(tk.Toplevel):
    """
    Ventana que muestra el progreso de un proceso con un texto y una PantallaProgreso.

    Atributos:
        - progreso (int)
        - barra (tk.ttk.Progressbar)

    Argumentos:
        - master (tk.Toplevel): ventana de la que será modal esta ventana.
        - maximo (int): máximo de pasos en el progreso.
        - accion (str): texto que describe el proceso.
        - modo (str): {'determinate','indeterminate'} permite designar si la barra de progreso funciona en modo 'determinate' (avanzando un número de pasos marcado) o 'indeterminate' (cada delta segundos la barra se mueve hacia delante una unidad).
    """
    def __init__(self,master,maximo,accion,modo):
        tk.Toplevel.__init__(self)
        self.resizable(False, False)

        frame = tk.Frame(self)
        frame.grid(padx=10,pady=10)

        self.__progreso = tk.IntVar()
        self.__progresoLabel = tk.IntVar()
        self.__progresoLabel.set(1)

        self.__barra = ttk.Progressbar(frame, mode=modo,maximum=maximo,variable=self.__progreso)
        self.__barra.grid(row=1,column=0,columnspan=3,pady=5)

        if(modo == 'determinate'):
            ttk.Label(frame,text=accion).grid(row=0,column=0)
            labelProgresoParseado = ttk.Label(frame,textvariable=self.__progresoLabel)
            labelProgresoParseado.grid(row=0,column=1)
            ttk.Label(frame,text=" de %d" % maximo).grid(row=0,column=2)
        else:
            ttk.Label(frame,text=accion).grid(row=0,column=0,columnspan=3)

        self.grab_set()
        #self.wm_attributes("-topmost",1)
        self.focus()
        self.transient(master=master)
        #no permitimos cerrarla
        self.protocol("WM_DELETE_WINDOW", self.__nopermitir)

    def __nopermitir(self):
	    pass
		
    def step(self,step):
        """
        Avanza el progreso `step` unidades. Solo en modo 'determinate'.

        Argumentos:
            - step (int)
        """
        self.__progreso.set(self.__progreso.get()+step)
        self.__progresoLabel.set(self.__progresoLabel.get()+step)

    def start(self,delta=1):
        """
        Comienza a mover la barra. Solo en modo 'indeteminate'

        Argumentos:
            - delta (float) cada cuantos segundos se mueve la barra.
        """
        self.__barra.start(delta)

    def setProgreso(self,progreso):
        """
        Establece el progreso actual del proceso al valor dado.

        Argumentos:
            - progreso (int)
        """
        self.__progreso.set(progreso)
        self.__progresoLabel.set(progreso+1)


class SelectorModelo(tk.Frame):
    """
    Lista de selección de modelos con detalles.

    Argumentos:
        - master (Tkinter.Toplevel/Tkinter.Frame): Ventana o frame en el que insertar el elemento.

    Atributos:
        - datos_modelos (list:modelo.Modelo)
    """
    def __init__(self, master):
        tk.Frame.__init__(self,master)
        self.parent = master

        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(0,weight=1)

        frame = ttk.LabelFrame(self,text="Modelos guardados")
        frame.grid(row=0,column=0,sticky='nsew')
        frame.grid_columnconfigure(0,weight=1)

        frame.grid_rowconfigure(0,weight=1)

        vscrollbar = tk.Scrollbar(frame)

        self.__tabla = tk.Listbox(frame,yscrollcommand=vscrollbar.set,selectmode=tk.BROWSE)
        self.__tabla.grid(column=0, row=0,padx=10,pady=10, sticky='nsew')


        vscrollbar.config(command=self.__tabla.yview)
        vscrollbar.grid(row=0,column=1,sticky='nsew',in_=frame)

        # Detalles del modelo seleccionado
        frame_detalles = ttk.Frame(frame)
        frame_detalles.grid(column=2,row=0,sticky='nsew',padx=20)

        tipoLabel = ttk.Label(frame_detalles,text="Tipo de Red:")
        tipoLabel.grid(row=0,column=0,sticky='e')
        self.__tipoVar = tk.StringVar()
        tipoValue = ttk.Label(frame_detalles,textvariable=self.__tipoVar)
        tipoValue.grid(row=0,column=2,sticky='e')

        neuronasLabel = ttk.Label(frame_detalles,text="Neuronas en la capa oculta:")
        neuronasLabel.grid(row=1,column=0,sticky='e')
        self.__neuronasVar = tk.IntVar()
        neuronasValue = ttk.Label(frame_detalles,textvariable=self.__neuronasVar)
        neuronasValue.grid(row=1,column=2,sticky='e')

        clasesLabel = ttk.Label(frame_detalles,text="Clases posibles:")
        clasesLabel.grid(row=2,column=0,sticky='e')
        self.__clasesVar = ttk.Frame(frame_detalles)
        self.__clasesVar.grid(row=2,column=2,sticky='e')

    def muestraModelos(self,datos):
        """
        Inserta los modelos en la lista.

        Argumentos:
            - datos (list:modelo.Modelo)
        """
        self.datos_modelos = datos
        for m in datos:
            self.__tabla.insert('end',m.getNombre())

        # seleccionar el primer elemento por defecto
        self.__tabla.bind('<<ListboxSelect>>', self.__eventoSelItem)
        self.__tabla.selection_set(0)
        self.__tabla.event_generate("<<ListboxSelect>>") #hacemos que se actualice al primer elemento

    def __eventoSelItem(self,evento):
        indiceEnLista = self.__tabla.curselection()[0]

        self.__tipoVar.set(self.datos_modelos[indiceEnLista].getTipoRed())
        self.__neuronasVar.set(self.datos_modelos[indiceEnLista].getNumeroNeuronasOculta())
        listaClases = self.datos_modelos[indiceEnLista].getClases()

        # limpiamos las labels dentro de clasesVar
        for label in self.__clasesVar.winfo_children():
            label.destroy()

        for i in range(0,len(listaClases)):
            label_aux = ttk.Label(self.__clasesVar,text=listaClases[i])
            label_aux.grid(row=0,column=i,padx=5)

    def getSeleccion(self):
        """
        Devuelve el modelo seleccionado actualmente.

        Devuelve:
            - modelo (modelo.Modelo)
        """
        return self.datos_modelos[self.__tabla.curselection()[0]]
