# -*- coding: utf-8 -*-
"""
Created on Mon Mar 12 10:40:55 2018

@author: Esther Cuervo Fernández
"""
import sqlite3
from Modelo import modelo
import os
import sys

class DAO:
    """
    Clase que actúa como Fachada del resto de clases del paquete.

    Atributos:
        - fichero_bd (str): ruta al fichero de base de datos.
        - flagDBAbierta (boolean): flag que controla que solo pueda haber una conexión a la base de datos cada vez.
        - conexion (sqlite3.Connection)
        - fdao (DocumentoDAO): Instancia del Data Object que representa Documento
        - mdao (ModeloDAO): Instancia del Data Object que representa Modelo
        - cursor (sqlite3.Cursor)
    """
    def __init__(self):
		# la base de datos se encuentra en el directorio DB un directorio por encima del actual
        self.fichero_bd = os.path.join(os.path.split(os.path.dirname(__file__))[0],"DB","fichero_bd.sqlite")
        self.flagBDAbierta = False
        self.conexion = None
        self.fdao = DocumentoDAO()
        self.mdao = ModeloDAO()
        self.cursor = None

    def conectar(self):
        """
        Abre la conexión a la base de datos.
        """
        if(self.conexion == None or not(self.flagBDAbierta)):
            self.conexion = sqlite3.connect(self.fichero_bd)
            self.flagBDAbierta = True
            self.cursor = self.conexion.cursor()

    def close(self):
        """
        Cierra la conexión a la base de datos.
        """
        if(self.conexion != None):
            self.flagBDAbierta = False
            self.conexion.commit()
            self.conexion.close()
            self.conexion = None
            self.cursor = None

    def getDocumentos(self):
        """
        Devuelve todos los documentos existentes en el sistema.

        Devuelve:

            - output (list:modelo.Documento)
        """
        values = self.fdao.getDocumentos(self.cursor)

        output = list()
        for f in values:
            fa = modelo.Documento()
            fa.setId(f[0])
            fa.setTitulo(f[1])
            fa.setClasificacion(f[2])
            fa.setCuerpo(f[3])

            output.append(fa)

        return output

    def getDocumento(self,iddocumento):
        """
        Devuelve el documento del sistema con identificador `iddocumento`.

        Argumentos:

            - iddocumento (str)

        Devuelve:

            - fa (modelo.Documento)

        Excepciones:

            - ValueError: la función lanza esta excepción si no existe el documento con identificador `iddocumento`.
        """
        f = self.fdao.getDocumento(self.cursor,iddocumento)

        if(len(f)!=0):
            f = f[0]
        else:
            raise ValueError("No existe el documento con id %d" % iddocumento)

        fa = modelo.Documento()
        fa.setId(f[0])
        fa.setTitulo(f[1])
        fa.setClasificacion(f[2])
        fa.setCuerpo(f[3])

        return fa

    def uploadDocumentos(self,documentos):
        """
        Almacena los documentos en el sistema.

        Argumentos:
            - documentos (list:modelo.Documento)
        Devuelve:
            - documentos (list:modelo.Documento): lista de documentos en argumentos actualizada con el identificador del documento.
        """
        for documento in documentos:
            idf = self.fdao.newDocumento(self.cursor,documento.getTitulo(),documento.getClasificacion(),documento.getCuerpo())
            documento.setId(idf)
        self.conexion.commit()
        return documentos

    def deleteDocumentos(self,documentos):
        """
        Elimina los documentos del sistema.

        Argumentos:
            - documentos (list:modelo.Documento)
        """
        for documento in documentos:
            self.fdao.deteleDocumento(self.cursor,documento.getId())
        self.conexion.commit()

    def getModelos(self):
        """
        Devuelve todos los modelos almacenados en el sistema.

        Devuelve:

            - modelos (list:modelo.Modelo)
        """
        values = self.mdao.getModelos(self.cursor)
        output = list()

        for m in values:
            mo = modelo.Modelo()
            mo.setId(m[0])
            mo.setNombre(m[1])
            mo.setNumeroNeuronasOculta(m[2])
            clases = self.mdao.getClases(self.cursor,m[0])
            mo.setClases(clases)
            mo.setMaximoNormalizado(m[3])
            mo.setMinimoNormalizado(m[4])
            nombre = self.mdao.getNombreTipoRed(self.cursor,m[5])
            if(nombre == None):
                raise ValueError("No existe el tipo de red con id %d" % m[5])
            else:
                mo.setTipoRed(nombre[0])

                pesos,bias = self.__getPesos(mo.getId())
                mo.setPesos(pesos)
                mo.setBias(bias)

                mo.setVocabulario(dict(self.mdao.getVocabulario(self.cursor,mo.getId())))

                output.append(mo)

        return output

    def getModelo(self,idModelo):
        """
        Devuelve el modelo almacenado en el sistema con identificador `idModelo`.

        Devuelve:

            - modelos (list:modelo.Modelo)

        Excepciones:

            - ValueError: la función lanza esta excepción si no existe el modelo con identificador `idModelo`.
        """
        m = self.mdao.getModelo(self.cursor,idModelo)
        if(len(m)!=0):
            m = m[0]
        else:
            raise ValueError("No existe el modelo con id %d" % idModelo)
        mo = modelo.Modelo()
        mo.setId(idModelo)
        mo.setNombre(m[1])
        mo.setNumeroNeuronasOculta(m[2])
        clases = self.mdao.getClases(self.cursor,idModelo)
        mo.setClases(clases)
        mo.setMaximoNormalizado(m[3])
        mo.setMinimoNormalizado(m[4])
        nombreRed = self.mdao.getNombreTipoRed(self.cursor.m[5])
        if(nombreRed==None):
            raise ValueError("No existe el tipo de red con id %d" % m[5])
        else:
            mo.setTipoRed(nombreRed[0])

            pesos,bias = self.__getPesos(mo.getId())
            mo.setPesos(pesos)
            mo.setBias(bias)

            mo.setVocabulario(dict(self.mdao.getVocabulario(self.cursor,idModelo)))

            return mo


    def uploadModelo(self,modelo):
        """
        Almacena el modelo en el sistema.

        Argumentos:

            - modelo (modelo.Modelo)
        """
        # convertimos el tipo de red
        idTipoRed = self.mdao.getIdTipoRed(self.cursor,modelo.getTipoRed().upper())
        if(idTipoRed == None):
            raise ValueError("No existe el tipo de red con nombre %s" % modelo.getTipoRed())
        else:
            idModelo = self.mdao.newModelo(self.cursor,modelo.getNombre(), modelo.getNumeroNeuronasOculta(), idTipoRed[0], modelo.getMaximoNormalizado(), modelo.getMinimoNormalizado())

            pesos = modelo.getPesos()
            bias = modelo.getBias()

            vocab = modelo.getVocabulario()
            clases = modelo.getClases()


            for k in pesos:
                for i,p in enumerate(pesos[k]):
                    for j,v in enumerate(p):
                        self.mdao.newPeso(self.cursor,idModelo,k,i,j,v)

            #el bias se guarda como otro peso normal, pero la neurona out es -1
            for k in bias:
                for i,p in enumerate(bias[k]):
                    for j,v in enumerate(p):
                        self.mdao.newPeso(self.cursor,idModelo,k,-1,j,v)


            for c in clases:
                self.mdao.newClase(self.cursor,idModelo,c)

            for k in vocab:
                self.mdao.newPalabraVocabulario(self.cursor,idModelo,k,vocab[k])

            self.conexion.commit()

    def __getPesos(self,idModelo):
        rawPesos = self.mdao.getPesos(self.cursor,idModelo)
        pesos = dict()
        bias = dict()

        capas = set([x[0] for x in rawPesos])
        for nombre in capas:
                raw = [x for x in rawPesos if x[0]==nombre]
                nombre = nombre.lower()
                #con el numero de neuronas podemos crear una lista vacía que vamos rellenando por posición
                neuronasIn = set([x[2] for x in raw])
                neuronasOut = set([x[1] for x in raw])
                p = []
                b = []
                for n in neuronasOut:
                    if(n!=-1):
                        p.append([0.0]*len(neuronasIn))
                if -1 in neuronasOut:
                    b.append([0.0]*len(neuronasIn))

                #rellenamos el array
                for _, nOut, nIn, valor in raw:
                    if(nOut != -1):
                        p[nOut][nIn] = valor
                    else:
                        b[0][nIn] = valor
                pesos[nombre] = p
                if -1 in neuronasOut:
                    bias[nombre] = b
        return pesos,bias

class DocumentoDAO:

    def getDocumentos(self,cursor):
        cursor.execute("SELECT * FROM DOCUMENTO")
        return cursor.fetchall()

    def getDocumento(self,cursor,idF):
        cursor.execute("SELECT * FROM DOCUMENTO F WHERE F.IDDOCUMENTO = ?",(idF,))
        return cursor.fetchall()

    def newDocumento(self,cursor,titulo,clasificacion,cuerpo):
        cursor.execute("INSERT INTO DOCUMENTO(TITULO,CLASIFICACION,CUERPO) VALUES(?,?,?)",(str(titulo),str(clasificacion),str(cuerpo),))
        return cursor.lastrowid

    def deteleDocumento(self,cursor,idF):
        cursor.execute("DELETE FROM DOCUMENTO WHERE IDDOCUMENTO=?",(idF,))

class ModeloDAO:

    def getModelos(self,cursor):
        cursor.execute("SELECT * FROM MODELO")
        return cursor.fetchall()

    def getModelo(self,cursor,iM):
        cursor.execute("SELECT * FROM MODELO M WHERE M.IDMODELO=?",(iM,))
        return cursor.fetchall()

    def getNombreTipoRed(self,cursor,indice):
        cursor.execute("SELECT T.NOMBRE FROM RED_TIPO T WHERE T.ID=?",(indice,))
        return cursor.fetchone()

    def getIdTipoRed(self,cursor,nombre):
        cursor.execute("SELECT T.ID FROM RED_TIPO T WHERE T.NOMBRE=?",(nombre,))
        return cursor.fetchone()

    def getPesos(self,cursor,indiceModelo):
        cursor.execute("SELECT P.CAPA, P.NEURONAOUT, P.NEURONAIN, P.VALOR FROM PESO P WHERE P.IDMODELO=?",(indiceModelo,))
        return cursor.fetchall()

    def getClases(self,cursor,indiceModelo):
        cursor.execute("SELECT C.VALOR FROM CLASE C WHERE C.IDMODELO=?",(indiceModelo,))
        return cursor.fetchall()

    def getVocabulario(self,cursor,indiceModelo):
        cursor.execute("SELECT V.PALABRA, V.VALOR FROM VOCABULARIO V WHERE V.IDMODELO=?",(indiceModelo,))
        return cursor.fetchall()

    def newModelo(self,cursor,nombre, numNeuronas, tipoRed, maximo, minimo):
        cursor.execute("INSERT INTO MODELO(NOMBRE, NUMERONEURONAS, TIPORED, MAXNORMALIZADO, MINNORMALIZADO) VALUES(?,?,?,?,?)",(str(nombre),int(numNeuronas),int(tipoRed),int(maximo), int(minimo),))
        return cursor.lastrowid

    def newPeso(self,cursor,idModelo, capa, neuronaOut, neuronaIn, valor):
        cursor.execute("INSERT INTO PESO(IDMODELO, CAPA, NEURONAOUT, NEURONAIN, VALOR) VALUES(?,?,?,?,?)",(int(idModelo),str(capa),int(neuronaOut),int(neuronaIn),float(valor),))
        return cursor.lastrowid

    def newClase(self,cursor,idModelo, valor):
        cursor.execute("INSERT INTO CLASE(IDMODELO, VALOR) VALUES(?,?)",(int(idModelo),str(valor),))
        return cursor.lastrowid

    def newPalabraVocabulario(self,cursor,idModelo, palabra, valor):
        cursor.execute("INSERT INTO VOCABULARIO(IDMODELO, PALABRA, VALOR) VALUES(?,?,?)",(int(idModelo),str(palabra),int(valor),))
        return cursor.lastrowid
