# -*- coding: utf-8 -*-

from nltk.stem import SnowballStemmer
from nltk.corpus import stopwords

import re

from numpy.random import seed, shuffle

class ConversorTextos:
    """
        Esta clase convierte textos complejos
        en un vector de números enteros para análisis de textos.

        Atributos:
            - vocab (dict:str->int): Asociacion 1-1 entre palabras y enteros.
            - tokens (list:list:int): Vector de enteros resultado.

        Argumentos:
            - idioma (str): permite denotar el idioma del texto, mediante el nombre del idioma en inglés, a partir del cual se elegiran las stopwords automáticas y el stemmer. (por defecto 'spanish')
            - stop (boolean): determina si se eliminan las palabras 'stopword' generadas automaticamente de la librería NTLK si se ha designado el idioma. Si no se designa idioma se utilizan solo la lista de extrastop. (por defecto True)
            - extrastop (list:str): lista de palabras extra a eliminar de los textos a la hora de tokenizar. (por defecto None)
            - orden (str): orden en el que las palabras se asignan a números en el vocabulario. {'LEXICOGRAFICO', 'ALEATORIO', 'APARICION'}. (por defecto 'APARICION')
            - semilla (int): si el orden elegido es aleatorio, semilla para el generador de números aleatorios. (por defecto None)
          Lanza:
            - OSError: esta clase lanza esta excepción si no existen stopwords o SnowballStemmer para el lenguaje seleccionado.
    """
    def __init__(self,idioma='spanish',stop=True,extrastop=None,orden='APARICION',semilla=None):

        self.stopwords = set([])

        if(stop and idioma):
            try:
                self.stopwords = set(stopwords.words(idioma))
            except OSError:
                raise OSError("Idioma %s no soportado" % idioma)
                return

            if(extrastop!=None):
                self.stopwords = self.stopwords.union(set(extrastop))

        try:
            self.stemmer = SnowballStemmer(idioma,ignore_stopwords=True)
        except OSError:
            raise OSError("Idioma %s no soportado" % idioma)
            return

        self.orden = orden
        self.semilla = semilla

        self.vocab = None
        self.tokens = None

        self.re_entero = re.compile('[0-9]+(\.[0-9]{3})*')
        self.re_fecha = re.compile('[0-9]+\/[0-9]+\/[0-9]+')
        self.re_real = re.compile('[0-9]+(\.[0-9]{3})*\,[0-9]+')
        if(idioma=='english'):
            self.re_noletras = re.compile("[^\'\w]|\_")   #para el inglés no eliminamos los apóstrofes
        else:
            self.re_noletras = re.compile("[\W\_]")

    def fit_transform(self,vector_textos):
        """
        Calcula los valores para el vocabulario a partir de una lista de textos complejos, y devuelve la lista con las palabras convertidas en números enteros.

        Este método también realiza un limpiado por stopwords, si estas se han activado en la creación del objeto, y elimina las palabras que solo aparecen en un solo documento.

        Argumentos:

            - vector_textos (list:str): array formado por strings de texto complejo

        Devuelve:

            - vector (list:int): array de palabras transformadas a números enteros o #UNK#
        """

        self.tokens = [self.tokenizado(self.limpia(x)) for x in vector_textos]
        self.tokens = self.reduccionPalabras([self.stemizar([item for item in sublist if item not in self.stopwords]) for sublist in self.tokens],1)
        self.vocab = self.vocabulario([item for sublist in self.tokens for item in sublist],self.orden,self.semilla)
        self.tokens = [[self.vocab[palabra] if palabra in self.vocab.keys() else '#UNK#' for palabra in doc] for doc in self.tokens]

        return self.tokens

    def transform(self,vector_textos):
        """
        Transforma el texto complejo a vector de números con el vocabulario almacenado.
        Si una palabra no aparece en el vocabulario, se sustituye por el token '#UNK#'.

        Argumentos:

            - vector_textos (list:str): array formado por strings de texto complejo

        Devuelve:

            - vector (list:str): array de palabras transformadas a números enteros o #UNK#
        """
        self.tokens = [self.tokenizado(self.limpia(x)) for x in vector_textos]
        self.tokens = [self.stemizar(x) for x in self.tokens]
        
        self.tokens = [[self.vocab[palabra] if palabra in self.vocab.keys() else '#UNK#' for palabra in doc] for doc in self.tokens]
        return self.tokens

    def limpia(self,texto):
        """
        Transforma un texto complejo a uno que solo contiene caracteres alfabéticos,
        está en minúsculas, y contiene NUMTOKEN en cada posición con números.


        Argumentos:

            - texto (str): Cadena de caracteres conteniendo el texto complejo

        Devuelve:

            - clean: (str): Cadena de caracteres con el texto limpiado
        """
        clean = self.re_fecha.sub(' DATETOKEN ',texto.lower())
        clean = self.re_real.sub(' REALTOKEN ',clean)
        clean = self.re_entero.sub(' INTTOKEN ',clean)
        clean = self.re_noletras.sub(' ',clean)
        return clean

    def tokenizado(self,texto):
        """
        Transforma un texto a un vector de palabras.

        Argumentos:

            - texto (str): Cadena de caracteres conteniendo el texto

        Devuelve:

            - vector (list:str): Array de palabras
        """
        return texto.split()

    def stemizar(self,palabras):
        """
        Realiza una transformación de palabra completa a la raíz o `stem` utilizando el algoritmo SnowballStemmer.

        Este stemmer ignora los tokens (INTTOKEN, REALTOKEN y DATETOKEN)

        Argumentos:

            - palabras (list:str): Lista de palabras

        Devuelve:

            - vectorStemmizado (list:str): Array de palabras reducidas a su raíz.
        """
        return [self.stemmer.stem(x) if x != 'INTTOKEN' and x != 'REALTOKEN' and x != 'DATETOKEN' else x for x in palabras]

    def reduccionPalabras(self,corpus,docMinimos):
        """
        Reduce el número de palabras superfluas calculando el número de documentos en los que aparece cada palabra del corpus, y eliminando aquellas con una frecuencia menor que la indicada en `docMinimos`.

        Este método devolverá el texto original si docMinimos es menor o igual que el número de ficheros en el corpus, lo cual provocaría que todos los textos estuvieran vacios.

        Argumentos:

            - corpus (list:list:str): Lista de documentos formados por una lista de palabras.
            - docMinimos (int): Número mínimos de documentos en los que debe estar una palabra para que se mantenga en el corpus.

        Devuelve:

            - vectorLimpiado (list:list:str): Lista de documentos formados por una lista de palabras sin las palabras que ocurren en menos de `docMinimos` documentos.
        """

        if(len(corpus)<=docMinimos):
            return corpus
        else:
            #set de palabras únicas en cada documento
            set_palabras_por_doc = [set(palabras) for palabras in corpus]
            #set de todas las palabras en el corpus
            palabras_en_total = set([item for sublist in corpus for item in sublist])

            #número de documentos
            num_documentos = dict()
            for palabra in palabras_en_total:
                num_documentos[palabra] = len([1 for s in set_palabras_por_doc if palabra in s])

            return [[item for item in sublist if num_documentos[item]>docMinimos] for sublist in corpus]




    def vocabulario(self,palabras,orden,semilla):
        """
        Genera una asociación 1-1 entre palabras y números enteros.

        Argumentos:

            - palabras (list:str): Lista de palabras
            - orden:   (str): Designa el orden en el que asignar números a las palabras.  {'LEXICOGRAFICO', 'ALEATORIO','APARICION'}
            - semilla (int): Designa la semilla aleatoria a utilizar en el caso de usar un orden aleatorio.
            - frecuenciaMinima (int): Designa la frecuencia con la que tiene que aparecer una palabra como mínimo en el corpus para considerarla en el vocabulario.

        Devuelve:

            - vocabulario (dict:str->int): Diccionario con la asociación 1-1 de palabra->valor

        Excepciones:

            - ValueError: esta función lanza esta excepción en el caso de utilizar un orden distinto a los descritos en este docstring.
        """
        orden = orden.upper()
        if(orden=='LEXICOGRAFICO'):
            uniq = sorted(list(set(palabras)))
            vocabulario = dict((palabra,num) for (num,palabra) in enumerate(uniq))

        elif(orden=='ALEATORIO' or orden=='APARICION'):
            if(orden=='ALEATORIO'):
                seed(semilla)
                shuffle(palabras)
            vocabulario = dict()
            num = 0
            for x in palabras:
                if x not in vocabulario.keys():
                    vocabulario[x] = num
                    num+=1
        else:
            raise ValueError("No existe implementación para ordenación del vocabulario %s" % orden)

        return vocabulario

    def getVocabulario(self):
        """
        Devuelve la asociación palabra->valor almacenado tras aplicar fit_transform.

        Devuelve:

            - vocabulario (dict:str->int): Diccionario con la asociación 1-1 de palabra->valor
        """
        return self.vocab

    def getVector(self):
        """
        Devuelve la lista de vectores de enteros almacenados tras aplicar fit_transform/transform.

        Devuelve:

            - tokens: (list:list:int): Lista de vectores numéricos
        """
        return self.tokens

    def setVocabulario(self,vocabulario):
        """
        Establece el vocabulario indicado como asociación de palabra->valor a usar por el conversor.

        Argumentos:

            - vocabulario (dict:str->int): Diccionario con la asociación 1-1 de palabra->valor
        """
        self.vocab = vocabulario
