# -*- coding: utf-8 -*-
"""
Este fichero incluye la interfaz a implementar por todas las redes, y la implementación de las redes Elman y Jordan.

@author Esther Cuervo Fernández
"""
from Redes.funciones import sigmoide, dsigmoide, inicializaPeso
from numpy import array, zeros, matmul, unique, concatenate, where, full, argmax
from numpy.random import seed

class Red:
    """
    Interfaz que deben implementar todas las redes.
    """

    def entrena(self,X,Y,X_val,Y_val):
        """
        Inicializa los valores iniciales necesarios para comenzar el entrenamiento de la red.

        Argumentos:

            - X (array: list: float): Vector de instancias utilizadas para entrenar la red.
            - Y (array: str): Vector de etiquetas de clase para entrenamiento.
            - X_val (array: list: float): Vector de instancias utilizadas para validar la red.
            - Y_val (array:str): Vector de eitquetas de clase para validación.

        Excepciones:

            - ValueError: la función lanza esta excepción si el número de clases posibles en
            el conjunto de entrenamiento no coincide con el número de neuronas de salida.
        """
        pass

    def epoca(self):
        """
        Realiza una época del entrenamiento, iterando por todas las instancias de entrenamiento de la red y aplicando
        la variación de pesos para esa época de entrenamiento.
        """
        pass

    def clasifica(self,X):
        """
        Devuelve las etiquetas obtenidas al aplicar el estado actual de la red a X.

        Argumentos:

            - X (array: list: float): Vector de instancias a clasificar.

        Devuelve:

            - Y (array: str): Vector de etiquetas correspondientes a cada instancia.
        """
        pass

    def score(self,X,Y):
        """
        Devuelve la matriz de confusión resultado de comparar las etiquetas obtenidas de clasificar X
        y las etiquetas reales Y.

        Argumentos:

            - X (array: list: float): Vector de instancias.
            - Y (array: str): Vector de etiquetas reales de cada instancia.

        Devuelve:

            - C (array: int): Matriz de confusión con forma nxm, siendo n el número de etiquetas distintas en Y, y m el número de etiquetas que puede clasificar la red. Cij = número de instancias de clase real i clasificadas como j
            - etiquetasFilas (array: str): Vector de etiquetas correspondientes a cada fila de la matriz.
            - etiquetasColumnas (array: str): Vector de etiquetas correspondientes a cada columna de la matriz.
        """

    def setPesos(self,pesos,bias):
        """
        Permite asignar pesos y bias de la red manualmente. Útil para realizar clasificación sin previo entrenamiento.
        Para los pesos se sigue la siguiente notación:
                Wij = peso de la conexión que va desde la neurona ui a la uj.
                       En los casos donde exiten capas, ui está en H, y uj en H+1.
                       En los casos donde haya retroalimentación, u_i existe en el instante t-1 y u_j en el t.
                La clave indica la capa de ui-uj (ejemplo: entrada-oculta y oculta-salida)
        Para los bias se sigue la notación:
            Bi = bias de la neurona u_i.
            La clave que indica la capa en el diccionario debe ser la capa donde está la neurona ui para bias Bi.

        Argumentos:

            - pesos (dict): Diccionario con estructura capa->array(float) con los valores para los pesos. La forma de los pesos debe ser [M,N], con M el número de neuronas en la capa a la que pertenece u_i, y N el número de neuronas en la capa a la que pertenece u_j.
            - bias (dict): Diccionario con estructura capa->array(float) con los valores para los bias. La forma de los bias debe ser [1,N], siendo N el número de neuronas en la capa.

        Excepciones:

            - TypeError: la función lanza esta excepción si alguna de las claves de los pesos o de los bias no  corresponde con los pesos que tendría que tener la red.
            - ValueError: la función lanza esta excepción si la forma de algún peso no coincide con el número de neuronas de su capa.
        """
        pass

    def setClases(self,clases):
        """
        Permite asignar las etiquetas a utilizar como clases de la red. Útil para realizar clasificación sin previo
        entrenamiento.

        Argumentos:

            - clases (array:str)

        Excepciones:

            - ValueError: la función lanza esta excepción si el número de clases no coincide con el número de neuronas de salida.
        """
        pass

    def getPesos(self):
        """
        Devuelve diccionarios con pesos y bias de la red.
        Para los pesos se sigue la siguiente notación:
                Wij = peso de la conexión que va desde la neurona ui a la uj.
                       En los casos donde exiten capas, ui está en H, y uj en H+1.
                       En los casos donde haya retroalimentación, u_i existe en el instante t-1 y u_j en el t.
                La clave indica la capa de ui-uj (ejemplo: entrada-oculta, oculta-salida)
        Para los bias se sigue la notación:
            Bi = bias de la neurona u_i.
            La clave que indica la capa en el diccionario es la capa donde está la neurona ui para bias Bi.

        Devuelve:

            - pesos (dict): Diccionario con estructura capa->array(float) con los valores para los pesos. La forma de los pesos es [M,N], con M el número de neuronas en la capa a la que pertenece u_i, y N el número de neuronas en la capa a la que pertenece u_j.
            - bias (dict): Diccionario con estructura capa->array(float) con los valores para los bias. La forma de los bias es [1,N], siendo N el número de neuronas en la capa.
        """
        pass

    def getClases(self):
        """
        Devuelve las etiquetas que utiliza la red como clases.

        Devuelve:

            - clases (array:str)
        """
        pass

class Elman(Red):
    """
    Elman(numNeuronasOculta,numNeuronasSalida,numNeuronasEntrada,factorAprendizaje,factorInercia,semillaAleatoria)
        Implementacion de la Red Neuronal Recursiva Elman. Esta red tiene una estructura similar a la de un
        Multilayer Perceptron tradicional, pero los resultados de la capa oculta en cada instante son utilizados
        como entrada extra en el instante inmediatamente posterior.

    Argumentos:
        - numNeuronasOculta (int): Número de neuronas a utilizar en la capa oculta de la red.
        - numNeuronasSalida (int): Número de neuronas a utilizar en la capa oculta de la red. Este valor debería ser el número de clases posibles.
        - numNeuronasEntrada (int): Número de neuronas a utilizar en la capa de entrada de la red. Este valor debería ser el tamaño de cada elemento de la secuencia a examinar.
        - factorAprendizaje (float): Factor de aprendizaje a utilizar. Este factor es el que multiplica a la variación de pesos en la fase de retropropagación del error.
        - factorInercia (float): Factor de inercia a utilizar. Este factor es el que multiplica la variación de pesos obtenida de la instancia anterior a la actual.
        - semillaAleatoria (int): Semilla aleatoria para el generador de números aleatorios.
    """
    def __init__(self,numNeuronasOculta,numNeuronasSalida,numNeuronasEntrada,factorAprendizaje,factorInercia,semillaAleatoria):
        self.numNeuronasOculta = numNeuronasOculta
        self.numNeuronasAuxiliar = self.numNeuronasOculta
        self.numNeuronasEntrada = numNeuronasEntrada
        self.numNeuronasSalida = numNeuronasSalida

        self.__factorAprendizaje = factorAprendizaje
        self.__factorInercia = factorInercia

        self.pesos__ = dict()
        self.bias__ = dict()
        self.clases__ = None

        #Inicializamos pesos
        seed(semillaAleatoria)
        pesosEntradaAOculta    = inicializaPeso((self.numNeuronasEntrada,self.numNeuronasOculta))
        biasOculta     = inicializaPeso((1,self.numNeuronasOculta))
        pesosAuxiliarAOculta  = inicializaPeso((self.numNeuronasAuxiliar,self.numNeuronasOculta))
        pesosOcultaASalida    = inicializaPeso((self.numNeuronasOculta,self.numNeuronasSalida))
        biasSalida     = inicializaPeso((1,self.numNeuronasSalida))

        self.pesos__ = {'entrada-oculta': pesosEntradaAOculta}
        self.bias__  = {'oculta': biasOculta}

        self.pesos__['auxiliar-oculta'] = pesosAuxiliarAOculta
        self.pesos__['oculta-salida'] = pesosOcultaASalida
        self.bias__['salida'] = biasSalida

        #inicializamos las inercias a 0
        self.__inerciaPesos = {'entrada-oculta': zeros((self.pesos__['entrada-oculta'].shape)),
                          'auxiliar-oculta':zeros((self.pesos__['auxiliar-oculta'].shape)),
                          'oculta-salida':zeros((self.pesos__['oculta-salida'].shape))}

        self.__inerciaBias = {'oculta': zeros((self.bias__['oculta'].shape)),
                              'salida': zeros((self.bias__['salida'].shape))}

    def entrena(self,X,Y,X_val,Y_val):
        Y = array(Y)
        X = array(X)
        X_val = array(X_val)
        Y_val = array(Y_val)

        self.clases__ = unique(Y)
        if(len(self.clases__)!=self.numNeuronasSalida):
            raise ValueError("El número de clases distintas en el conjunto de entrenamiento no coincide con el número de neuronas de salida numClases = %d numNeuronas = %d" % (len(self.clases__),self.numNeuronasSalida))
        self.clases__.sort()
        self.clases__ = array(self.clases__)

        self.X = X
        self.X_val = X_val

        #transformamos las Y en matrices con una fila por instancia
        #y una columna por cada clase posible, con 0.1 en todas
        #las posiciones excepto la de la clase correspondiente,
        #donde tendrá un 0.9
        Y_transformada = full((len(Y),len(self.clases__)),0.1)
        for i,c in enumerate(Y):
            Y_transformada[i][self.clases__ == c] = 0.9

        self.Y = Y_transformada

        Y_transformada = full((len(Y_val),len(self.clases__)),0.1)
        for i,c in enumerate(Y_val):
            Y_transformada[i][self.clases__ == c] = 0.9

        self.Y_val = Y_transformada


    def epoca(self):
        for i,instancia in enumerate(self.X):

            self.salidaOculta__ = zeros((1,self.numNeuronasOculta))
            self.__faseHaciaDelante(instancia)

            self.__faseHaciaAtras(self.Y[i])

    def clasifica(self,X):
        clas_output = []
        for instancia in X:

            self.salidaOculta__ = zeros((1,self.numNeuronasOculta))

            self.__faseHaciaDelante(instancia)
            #la salida de la red será un vector con tamaño numero_de_clases,
            #la clase elegida es el indice de la neurona con mayor valor
            clas_output.append(self.clases__[argmax(self.salidaSalida__)])

        return clas_output

    def score(self,X,Y):
        salidas_obtenidas = self.clasifica(X)

        self.clasesScore__ = unique(Y)
        self.clasesScore__ = array(sorted(unique(concatenate((self.clases__,self.clasesScore__)))))

        conf_matrix = zeros((len(self.clasesScore__),len(self.clases__)),dtype=int)
        for k,obt in enumerate(salidas_obtenidas):
            j, = where(self.clases__ == obt)[0]
            i, = where(self.clasesScore__ == Y[k])[0]
            conf_matrix[i][j] += 1

        return (conf_matrix, self.clasesScore__, self.clases__)

    def __faseHaciaDelante(self,instancia):
        #instancia: array con forma (tamano_instancia)
        for elemento in instancia:
            self.salidaAuxiliar__ = self.salidaOculta__

            self.salidaEntrada__ = array(elemento).reshape(1,-1)

            self.salidaOculta__ = matmul(self.salidaEntrada__ ,self.pesos__['entrada-oculta'])+matmul(self.salidaAuxiliar__,self.pesos__['auxiliar-oculta'])+self.bias__['oculta']
            self.salidaOculta__ = sigmoide(self.salidaOculta__)

            self.salidaSalida__ = matmul(self.salidaOculta__,self.pesos__['oculta-salida'])+self.bias__['salida']
            self.salidaSalida__ = sigmoide(self.salidaSalida__)

    def __faseHaciaAtras(self,salidaDeseada):
        self.delta__ = {}

        self.delta__['salida'] = (salidaDeseada-self.salidaSalida__)*dsigmoide(self.salidaSalida__)
        self.delta__['oculta'] = matmul(self.delta__['salida'],self.pesos__['oculta-salida'].T)*dsigmoide(self.salidaOculta__)

        variacionSalida = matmul(self.salidaOculta__.T,self.delta__['salida'])
        variacionBiasSalida = self.delta__['salida']

        variacionOculta = matmul(self.salidaEntrada__.T,self.delta__['oculta'])
        variacionBiasOculta = self.delta__['oculta']

        variacionAuxiliar = matmul(self.salidaAuxiliar__.T,self.delta__['oculta'])


        self.pesos__['entrada-oculta'] += self.__factorAprendizaje * variacionOculta +  self.__factorInercia * self.__inerciaPesos['entrada-oculta']
        self.pesos__['auxiliar-oculta'] += self.__factorAprendizaje * variacionAuxiliar +  self.__factorInercia * self.__inerciaPesos['auxiliar-oculta']
        self.pesos__['oculta-salida'] += self.__factorAprendizaje * variacionSalida +  self.__factorInercia * self.__inerciaPesos['oculta-salida']

        self.bias__['oculta'] += self.__factorAprendizaje * variacionBiasOculta +  self.__factorInercia * self.__inerciaBias['oculta']
        self.bias__['salida'] += self.__factorAprendizaje * variacionBiasSalida +  self.__factorInercia * self.__inerciaBias['salida']

        self.__inerciaPesos['entrada-oculta'] = variacionOculta
        self.__inerciaPesos['oculta-salida'] = variacionSalida
        self.__inerciaPesos['auxiliar-oculta'] = variacionAuxiliar
        self.__inerciaBias['oculta'] = variacionBiasOculta
        self.__inerciaBias['salida'] = variacionBiasSalida

    def getPesos(self):
        return (self.pesos__,self.bias__)

    def getClases(self):
        return self.clases__

    def setPesos(self,pesos,bias):
        pesosKeys = pesos.keys()
        biasKeys = bias.keys()
        if('entrada-oculta' not in pesosKeys or 'oculta-salida' not in pesosKeys or 'auxiliar-oculta' not in pesosKeys):
            raise TypeError("Formato de los pesos: %s incorrecto" % pesosKeys)
        elif('oculta' not in biasKeys or 'salida' not in biasKeys):
            raise TypeError("Formato de los pesos del bias: %s incorrecto" % biasKeys)
        else:
            for k in pesosKeys:
                self.pesos__[k] = array(pesos[k])
            for k in biasKeys:
                self.bias__[k] = array(bias[k])

            #pesos__[entrada-oculta].shape[0] = nEntrada .shape[1] = nOculta
            #pesos__[auxiliar-oculta].shape[0] = .shape[1] = nOculta
            #pesos__[oculta-salida].shape[0] = nOculta .shape[1] = nSalida
            if(self.pesos__['entrada-oculta'].shape!=(self.numNeuronasEntrada,self.numNeuronasOculta)
               or self.pesos__['auxiliar-oculta'].shape!=(self.numNeuronasAuxiliar,self.numNeuronasOculta)
               or self.pesos__['oculta-salida'].shape!=(self.numNeuronasOculta,self.numNeuronasSalida)):
                raise ValueError("Forma de los pesos incorrecta")
            elif(self.bias__['oculta'].shape!=(1,self.numNeuronasOculta) or self.bias__['salida'].shape!=(1,self.numNeuronasSalida)):
                raise ValueError("Forma de los bias incorrecta")

    def setClases(self,clases):
        self.clases__ = clases
        if(len(self.clases__)!=self.numNeuronasSalida):
            raise ValueError("El número de clases no coincide con el número de neuronas de salida numClases = %d numNeuronas = %d" % (len(self.clases__),self.numNeuronasSalida))
        self.clases__.sort()
        self.clases__ = array(self.clases__)

class Jordan(Red):
    """
    Jordan(numNeuronasOculta,numNeuronasSalida,numNeuronasEntrada,factorAprendizaje,factorInercia,semillaAleatoria)
        Implementacion de la Red Neuronal Recursiva Jordan. Esta red tiene una estructura similar a la de un
        Multilayer Perceptron tradicional, pero los resultados de la capa de salida en cada instante son utilizados
        como entrada extra en el instante inmediatamente posterior.

    Argumentos:
        - numNeuronasOculta (int): Número de neuronas a utilizar en la capa oculta de la red.
        - numNeuronasSalida (int): Número de neuronas a utilizar en la capa oculta de la red. Este valor debería ser el número de clases posibles.
        - numNeuronasEntrada (int): Número de neuronas a utilizar en la capa de entrada de la red. Este valor debería ser el tamaño de cada elemento de la secuencia a examinar.
        - factorAprendizaje (float): Factor de aprendizaje a utilizar. Este factor es el que multiplica a la variación de pesos en la fase de retropropagación del error.
        - factorInercia (float): Factor de inercia a utilizar. Este factor es el que multiplica la variación de pesos obtenida  de la instancia anterior a la actual.
        - semillaAleatoria (int): Semilla aleatoria para el generador de números aleatorios.
    """
    def __init__(self,numNeuronasOculta,numNeuronasSalida,numNeuronasEntrada,factorAprendizaje,factorInercia,semillaAleatoria):
        self.numNeuronasOculta = numNeuronasOculta
        self.numNeuronasEntrada = numNeuronasEntrada
        self.numNeuronasSalida = numNeuronasSalida
        self.numNeuronasAuxiliar = self.numNeuronasSalida

        self.__factorAprendizaje = factorAprendizaje
        self.__factorInercia = factorInercia

        self.pesos__ = dict()
        self.bias__ = dict()
        self.clases__ = None

        #Inicializamos pesos
        seed(semillaAleatoria)
        pesosOculta    = inicializaPeso((self.numNeuronasEntrada,self.numNeuronasOculta))
        biasOculta     = inicializaPeso((1,self.numNeuronasOculta))
        pesosAuxiliar  = inicializaPeso((self.numNeuronasAuxiliar,self.numNeuronasOculta))
        pesosSalida    = inicializaPeso((self.numNeuronasOculta,self.numNeuronasSalida))
        biasSalida     = inicializaPeso((1,self.numNeuronasSalida))

        self.pesos__ = {'entrada-oculta': pesosOculta}
        self.bias__  = {'oculta': biasOculta}

        self.pesos__['auxiliar-oculta'] = pesosAuxiliar
        self.pesos__['oculta-salida'] = pesosSalida
        self.bias__['salida'] = biasSalida


        #inicializamos las inercias a 0
        self.__inerciaPesos = {'entrada-oculta': zeros((self.pesos__['entrada-oculta'].shape)),
                          'auxiliar-oculta':zeros((self.pesos__['auxiliar-oculta'].shape)),
                          'oculta-salida':zeros((self.pesos__['oculta-salida'].shape))}

        self.__inerciaBias = {'oculta': zeros((self.bias__['oculta'].shape)),
                              'salida': zeros((self.bias__['salida'].shape))}

    def entrena(self,X,Y,X_val,Y_val):
        Y = array(Y)
        X = array(X)
        X_val = array(X_val)
        Y_val = array(Y_val)
        self.clases__ = unique(Y)
        if(len(self.clases__)!=self.numNeuronasSalida):
            raise ValueError("El número de clases no coincide con el número de neuronas de salida numClases = %d numNeuronas = %d" % (len(self.clases__),self.numNeuronasSalida))
        self.clases__.sort()
        self.clases__ = array(self.clases__)

        self.X = X
        self.X_val = X_val

        #transformamos las Y en matrices con una fila por instancia
        #y una columna por cada clase posible, con 0.1 en todas
        #las posiciones excepto la de la clase correspondiente,
        #donde tendrá un 0.9

        Y_transformada = full((len(Y),len(self.clases__)),0.1)
        for i,c in enumerate(Y):
            Y_transformada[i][self.clases__ == c] = 0.9

        self.Y = Y_transformada

        Y_transformada = full((len(Y_val),len(self.clases__)),0.1)
        for i,c in enumerate(Y_val):
            Y_transformada[i][self.clases__ == c] = 0.9

        self.Y_val = Y_transformada

    def epoca(self):
        for i,instancia in enumerate(self.X):

            self.salidaEntrada__ = zeros((1,self.numNeuronasEntrada))
            self.salidaAuxiliar__ = zeros((1,self.numNeuronasAuxiliar))
            self.salidaOculta__ = zeros((1,self.numNeuronasOculta))
            self.salidaSalida__ = zeros((1,self.numNeuronasSalida))
            self.__faseHaciaDelante(instancia)

            self.__faseHaciaAtras(self.Y[i])


    def clasifica(self,X):
        clas_output = []
        for instancia in X:

            self.salidaEntrada__ = zeros((1,self.numNeuronasEntrada))
            self.salidaAuxiliar__ = zeros((1,self.numNeuronasAuxiliar))
            self.salidaOculta__ = zeros((1,self.numNeuronasOculta))
            self.salidaSalida__ = zeros((1,self.numNeuronasSalida))

            self.__faseHaciaDelante(instancia)
            #la salida de la red será un vector con tamaño numero_de_clases,
            #la clase elegida es el indice de la neurona con mayor valor
            clas_output.append(self.clases__[argmax(self.salidaSalida__)])

        return clas_output

    def score(self,X,Y):
        salidas_obtenidas = self.clasifica(X)

        self.clasesScore__ = unique(Y)
        self.clasesScore__ = array(sorted(unique(concatenate((self.clases__,self.clasesScore__)))))

        conf_matrix = zeros((len(self.clasesScore__),len(self.clases__)),dtype=int)
        for k,obt in enumerate(salidas_obtenidas):
            j, = where(self.clases__ == obt)[0]
            i, = where(self.clasesScore__ == Y[k])[0]
            conf_matrix[i][j] += 1

        return (conf_matrix, self.clasesScore__, self.clases__)

    def __faseHaciaDelante(self,instancia):
        #instancia: array con forma (tamano_instancia)
        self.salidaSalida__ = zeros((self.salidaSalida__.shape))
        for elemento in instancia:
            self.salidaAuxiliar__ = self.salidaSalida__
            self.salidaEntrada__ = array(elemento).reshape(1,-1)

            salidaOcultaEntrada = matmul(self.salidaEntrada__ ,self.pesos__['entrada-oculta'])
            salidaOcultaAuxiliar = matmul(self.salidaAuxiliar__,self.pesos__['auxiliar-oculta'])

            self.salidaOculta__ = salidaOcultaEntrada+salidaOcultaAuxiliar
            self.salidaOculta__ = self.salidaOculta__+self.bias__['oculta']

            self.salidaOculta__ = sigmoide(self.salidaOculta__)

            self.salidaSalida__ = matmul(self.salidaOculta__,self.pesos__['oculta-salida'])+self.bias__['salida']
            self.salidaSalida__ = sigmoide(self.salidaSalida__)


    def __faseHaciaAtras(self,salidaDeseada):
        self.delta__ = {}

        self.delta__['salida'] = (salidaDeseada-self.salidaSalida__)*dsigmoide(self.salidaSalida__)
        self.delta__['oculta'] = matmul(self.delta__['salida'],self.pesos__['oculta-salida'].T)*dsigmoide(self.salidaOculta__)

        variacionSalida = matmul(self.salidaOculta__.T,self.delta__['salida'])
        variacionBiasSalida = self.delta__['salida']

        variacionOculta = matmul(self.salidaEntrada__.T,self.delta__['oculta'])
        variacionBiasOculta = self.delta__['oculta']

        variacionAuxiliar = matmul(self.salidaAuxiliar__.T,self.delta__['oculta'])

        self.pesos__['entrada-oculta'] += self.__factorAprendizaje * variacionOculta +  self.__factorInercia * self.__inerciaPesos['entrada-oculta']
        self.pesos__['auxiliar-oculta'] += self.__factorAprendizaje * variacionAuxiliar +  self.__factorInercia * self.__inerciaPesos['auxiliar-oculta']
        self.pesos__['oculta-salida'] += self.__factorAprendizaje * variacionSalida +  self.__factorInercia * self.__inerciaPesos['oculta-salida']

        self.bias__['oculta'] += self.__factorAprendizaje * variacionBiasOculta +  self.__factorInercia * self.__inerciaBias['oculta']
        self.bias__['salida'] += self.__factorAprendizaje * variacionBiasSalida +  self.__factorInercia * self.__inerciaBias['salida']

        self.__inerciaPesos['entrada-oculta'] = variacionOculta
        self.__inerciaPesos['oculta-salida'] = variacionSalida
        self.__inerciaPesos['auxiliar-oculta'] = variacionAuxiliar
        self.__inerciaBias['oculta'] = variacionBiasOculta
        self.__inerciaBias['salida'] = variacionBiasSalida


    def getPesos(self):
        return (self.pesos__,self.bias__)

    def getClases(self):
        return self.clases__

    def setPesos(self,pesos,bias):
        pesosKeys = pesos.keys()
        biasKeys = bias.keys()
        if('entrada-oculta' not in pesosKeys or 'oculta-salida' not in pesosKeys or 'auxiliar-oculta' not in pesosKeys):
            raise TypeError("Formato de los pesos: %s incorrecto" % pesosKeys)
        elif('oculta' not in biasKeys or 'salida' not in biasKeys):
            raise TypeError("Formato de los pesos del bias: %s incorrecto" % biasKeys)
        else:
            for k in pesosKeys:
                self.pesos__[k] = array(pesos[k])
            for k in biasKeys:
                self.bias__[k] = array(bias[k])

            #pesos__[entrada-oculta].shape[0] = nEntrada .shape[1] = nOculta
            #pesos__[auxiliar-oculta].shape[0] = nSalida .shape[1] = nOculta
            #pesos__[oculta-salida].shape[0] = nOculta .shape[1] = nSalida
            if(self.pesos__['entrada-oculta'].shape!=(self.numNeuronasEntrada,self.numNeuronasOculta)
               or self.pesos__['auxiliar-oculta'].shape!=(self.numNeuronasAuxiliar,self.numNeuronasOculta)
               or self.pesos__['oculta-salida'].shape!=(self.numNeuronasOculta,self.numNeuronasSalida)):
                raise ValueError("Forma de los pesos incorrecta")
            elif(self.bias__['oculta'].shape!=(1,self.numNeuronasOculta) or self.bias__['salida'].shape!=(1,self.numNeuronasSalida)):
                raise ValueError("Forma de los bias incorrecta")

    def setClases(self,clases):
        self.clases__ = clases
        if(len(self.clases__)!=self.numNeuronasSalida):
            raise ValueError("El número de clases no coincide con el número de neuronas de salida numClases = %d numNeuronas = %d" % (len(self.clases__),self.numNeuronasSalida))
        self.clases__.sort()
        self.clases__ = array(self.clases__)

        self.numNeuronasSalida = len(self.clases__)
        self.numNeuronasAuxiliar = self.numNeuronasSalida
