# -*- coding: utf-8 -*-

import tkinter as tk
from tkinter import simpledialog
from tkinter import ttk

from Elementos_reutilizables_vista.elementos import ScrollableLabelFrame, PantallaProgreso, TextoDocumento, FramesSeleccion, SelectorModelo

from Modelo import modelo

import queue
import threading

class Main(tk.Tk):
    """
    Vista de la pantalla inicial. Muestra los documentos en el sistema, permite ver su contenido, y permite acceder a las distintas partes de la aplicación.

    Atributos:
        - controlador (controlador.ControladorInicio): Instancia del controlador que controla esta vista.
    """
    def __init__(self,controlador):
        tk.Tk.__init__(self)
        self.controlador = controlador

        self.title("TFG Clasificación de Textos")

        self.iconbitmap('icono.ico')
        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(0,weight=1)

        menubar = tk.Menu(self)

        nuevomenu = tk.Menu(menubar,tearoff=0)

        menubar.add_command(label="Importar fichero",command=self.controlador.importaFicheros)

        menubar.add_cascade(label="Experimento...",menu=nuevomenu)

        nuevomenu.add_command(label="Clasificación",command=self.controlador.experimentoClasificacion)
        nuevomenu.add_command(label="Entrenamiento",command=self.controlador.experimentoEntrenamiento)

        menubar.add_command(label="Salir",command=self.controlador.salir)

        self.config(menu=menubar)

        self.frame_documentos = ScrollableLabelFrame(self)
        self.frame_documentos.config(text="Documentos importados")
        self.frame_documentos.grid(sticky='nsew',padx=20,pady=10)
        self.frame_documentos.getTree().bind("<Double-ButtonRelease-1>",self.__seleccionaDocumento)
        self.frame_documentos.getTree().config(selectmode='extended')
        self.frame_documentos.setHeadings(["#","Título","Clasificación"])
        self.frame_documentos.getTree().column(0,width=50,stretch=False)
        self.frame_documentos.getTree().column(1,width=500,stretch=True)
        self.frame_documentos.getTree().column(2,width=300,stretch=True)

        eliminarDocumentos = ttk.Button(self,text="Eliminar documentos",command =self.__eliminarDocumentos)
        eliminarDocumentos.grid(sticky='e',padx=40,row=1,column=0,pady=5)

        self.modalBarra = PantallaProgreso(self,100,'Cargando documentos en el sistema.','indeterminate')
        self.modalBarra.start()

        self.queueResultadoCarga = queue.Queue()
        self.queueProgresoCarga = queue.LifoQueue()

        hiloControlador = threading.Thread(target=self.controlador.cargaDocumentosSistema,args=(self.queueResultadoCarga,self.queueProgresoCarga,))

        hiloControlador.daemon = True
        hiloControlador.start()

        self.after(100,self.__compruebaProgresoCarga)


    def muestraDocumentos(self,documentos):
        """
        Carga los documentos indicados en la lista de documentos principal.

        Argumentos:
            - documentos (list:modelo.Documento)
        """
        for f in documentos:
            self.frame_documentos.addDat([f.getId(),f.getTitulo(),f.getClasificacion()])

    def __compruebaProgresoCarga(self):
        try:
            progreso = self.queueProgresoCarga.get(0)
            if(progreso =='FIN'):
                documentos = list()
                for q in range(self.queueResultadoCarga.qsize()):
                    f = self.queueResultadoCarga.get(q)
                    documentos.append(f)

                self.muestraDocumentos(documentos)

                self.modalBarra.destroy()
        except queue.Empty:
            self.after(100,self.__compruebaProgresoCarga)

    def __seleccionaDocumento(self,evento):
        w = evento.widget
        datos_documento = w.item(w.selection())['values']
        doc = modelo.Documento()
        doc.setId(datos_documento[0])

        self.controlador.cargaContenidoDocumento(doc)

    def __eliminarDocumentos(self):
        tabla = self.frame_documentos.getTree()
        num = len(tabla.selection())
        if(num>0):
            decision = tk.messagebox.askokcancel("Eliminar documentos","Se van a eliminar "+str(num)+" documentos del sistema. Esta acción es irreversible. ¿Desea eliminar permanentemente los documentos del sistema?")
            if(decision):
                documentos_seleccionados = list()
                for it in tabla.selection():
                    documento = modelo.Documento()
                    documento.setId(tabla.item(it)['values'][0])
                    documentos_seleccionados.append(documento)

                self.queueProgreso = queue.LifoQueue()

                hiloControlador = threading.Thread(target=self.controlador.eliminaDocumentos,args=(documentos_seleccionados,self.queueProgreso))

                self.modalBarra = PantallaProgreso(self,100,'Eliminando '+str(num)+' documentos ','indeterminate')
                self.modalBarra.start()

                hiloControlador.daemon = True
                hiloControlador.start()

                self.after(100,self.__compruebaProgresoEliminado)

                for it in tabla.selection():
                    tabla.delete(it)

    def __compruebaProgresoEliminado(self):
        try:
            progreso = self.queueProgreso.get(0)
            if(progreso!='FIN'):
                self.after(100,self.__compruebaProgresoEliminado)
            else:
                self.modalBarra.destroy()
        except queue.Empty:
            self.after(100,self.__compruebaProgresoEliminado)

    def muestraContenidoDocumento(self,documento):
        """
        Crea una instancia del visor de documentos interno con clase elementosReutilizablesVista.TextoDocumento y carga el contenido del documento dado.

        Argumentos:
            - documento (modelo.Documento): documento a mostrar.
        """
        editor = TextoDocumento()
        editor.muestraContenidoDocumento(documento)


class Importa(tk.Toplevel):
    """
    Vista de la pantalla de importación. Accede al sistema de ficheros para pedir al usuario la selección de ficheros a parsear, y tras el parseo, muestra los títulos y clasificaciones leídos, y permite modificarlos.

    Atributos:
        - controlador (controlador.ControladorImporta): Instancia del controlador que controla esta vista.
        - listadocumentos (list: modelo.Documento): Lista de documentos obtenidos tras el parseo.
        - vistaPrincipal (vista.Main): Instancia de la vista principal de la aplicación
    """
    def __init__(self,controlador,vistaPrincipal):
        tk.Toplevel.__init__(self)
        self.vistaPrincipal = vistaPrincipal
        self.controlador = controlador

        self.iconbitmap('icono.ico')
        self.title("Importar ficheros")

        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(0,weight=1)
        self.grid_rowconfigure(1,weight=0)

        self.frame_documentos = ScrollableLabelFrame(self)
        self.frame_documentos.configure(text="Documentos importados")
        self.frame_documentos.getTree().config(selectmode='browse')
        self.frame_documentos.grid(sticky='nsew',padx=10,pady=10)

        #Formulario
        frame_form = ttk.Frame(self)
        frame_form.grid(column=0,row=1,sticky='snew',padx=30,pady=20)

        frame_form.grid_columnconfigure(1,weight=1)

        titulo_label = ttk.Label(frame_form,text="Título:")
        titulo_label.grid(padx=10)

        self.titulo_valor = tk.StringVar()

        titulo_input = ttk.Entry(frame_form,exportselection=0,textvariable=self.titulo_valor)
        titulo_input.grid(column=1,row=0,padx=10,sticky='we')

        titulo_ast = ttk.Label(frame_form,text="(Opcional)")
        titulo_ast.grid(column=2,row=0)

        clasificacion_label = ttk.Label(frame_form,text="Clasificación:")
        clasificacion_label.grid(column=0,row=1,padx=10)

        self.clasificacion_valor = tk.StringVar()

        clasificacion_input = ttk.Entry(frame_form,exportselection=0,textvariable=self.clasificacion_valor)
        clasificacion_input.grid(column=1,row=1,padx=10,sticky='we')

        clas_ast = ttk.Label(frame_form,text="(Opcional)")
        clas_ast.grid(column=2,row=1)

        guardar_boton = ttk.Button(frame_form,text="Guardar atributos",command=self.__guardarAtributos)
        guardar_boton.grid(column=1,row=2,pady=10,sticky='e')

        # Botones
        frameBotones = ttk.Frame(self)
        frameBotones.grid(column=0,row=3,padx=20,pady=10,sticky='e')
        frameBotones.grid_columnconfigure(0,weight=1)
        frameBotones.grid_columnconfigure(1,weight=1)

        cancelar = ttk.Button(frameBotones,text="Cancelar", command=self.destroy)
        cancelar.grid(row=0,column=0,padx=10)

        siguiente = ttk.Button(frameBotones,text="Finalizar", command=self.__enviaDocumentos)
        siguiente.grid(row=0,column=1,padx=10)

        self.frame_documentos.getTree().bind('<ButtonRelease-1>', self.__eventoSelItem)
        self.frame_documentos.getTree().bind('<Double-ButtonRelease-1>', self.__muestraContenidoDocumento)
        self.frame_documentos.setHeadings(["Path","Título","Clasificación"])

    def pideFicheros(self):
        """
        Muestra el sistema de ficheros y permite al usuario seleccionar uno o más ficheros para el parseo.

        Reconoce las extensiones `.sgml` como fichero estructurado y cualquier otra extensión como fichero de texto plano.

        Durante el parseado, muestra el progreso.
        """
        ficheros_importados = tk.filedialog.askopenfilenames(title="Seleccione ficheros a parsear",
        filetypes=(("Ficheros estructurados sgml","*.sgml"),("Todos los ficheros","*.*")))

        self.longitudLista = len(ficheros_importados)
        #creamos las queue compartidas con el hilo del controlador
        self.queueDocumentos = queue.Queue()
        self.queueProgreso = queue.LifoQueue()

        if(ficheros_importados):
            self.listadocumentos = list()
            hiloControlador = threading.Thread(target=self.controlador.parseaFicheros,args=(ficheros_importados,self.queueDocumentos,self.queueProgreso))

            self.modalBarra = PantallaProgreso(self,self.longitudLista,"Parseando fichero ",'determinate')
            self.modalBarra.title("Parseado...")

            hiloControlador.daemon = True
            hiloControlador.start()

            self.after(100,self.__compruebaProgresoParseo)

        else:
            self.destroy()

    def __compruebaProgresoParseo(self):
        """
        Accede a las queues compartidas entre la vista y el hilo realizando el parseado y actualiza la barra de progreso y carga los documentos obtenidos si ha habido cambios.
        """
        try:
            progreso = self.queueProgreso.get(0)
            if(str(progreso).split(' ')[0] == 'ERRORSGML'):
                self.modalBarra.destroy()
                tk.messagebox.showerror("Fichero mal formado","El fichero %s no tiene la estructura correcta."%' '.join(progreso.split(' ')[1:]))
                self.destroy()
            elif(str(progreso).split(' ')[0] == 'ERRORPLANO'):
                self.modalBarra.destroy()
                tk.messagebox.showerror("Fichero no plano","El fichero %s no se puede leer como texto plano."%' '.join(progreso.split(' ')[1:]))
                self.destroy()
            else:
                self.modalBarra.setProgreso(progreso)
                for q in range(self.queueDocumentos.qsize()):
                    f = self.queueDocumentos.get(q)
                    self.listadocumentos.append(f)
                    self.frame_documentos.addDat([f.getPath(),f.getTitulo(),f.getClasificacion()])
                if(progreso<self.longitudLista-1):
                    self.after(100,self.__compruebaProgresoParseo)
                else:
                    self.modalBarra.destroy()
                    for q in range(self.queueDocumentos.qsize()):
                        f = self.queueDocumentos.get(q)
                        self.listadocumentos.append(f)
                        self.frame_documentos.addDat([f.getPath(),f.getTitulo(),f.getClasificacion()])
        except queue.Empty:
            self.after(100,self.__compruebaProgresoParseo)

    def __enviaDocumentos(self):
        #creamos un hilo para guardar cambios
        #creamos la queue de progreso y de resultado
        self.queueProgreso = queue.LifoQueue()
        self.queueDocumentos = queue.Queue()

        self.modalBarra = PantallaProgreso(self,100,"Guardando %d documentos "%len(self.listadocumentos),'indeterminate')
        self.modalBarra.title("Almacenando...")
        self.modalBarra.start()

        hiloControlador = threading.Thread(target=self.controlador.guardarCambios,args=(self.listadocumentos,self.queueProgreso,self.queueDocumentos))
        hiloControlador.daemon = True
        hiloControlador.start()
        self.after(100,self.__compruebaProgresoGuardado)


    def __compruebaProgresoGuardado(self):
        try:
            progreso = self.queueProgreso.get(0)
            if(progreso!='FINAL'):
                self.after(100,self.__compruebaProgresoParseo)
            else:
                self.modalBarra.destroy()

                nuevos_documentos = list()
                for q in range(self.queueDocumentos.qsize()):
                    f = self.queueDocumentos.get(q)
                    nuevos_documentos.append(f)

                #mandamos a la vista principal actualizar los documentos
                self.vistaPrincipal.muestraDocumentos(nuevos_documentos)

                self.destroy()
        except queue.Empty:
            self.after(100,self.__compruebaProgresoGuardado)

    def __muestraContenidoDocumento(self,event):
        w = event.widget
        it = w.item(w.selection())
        path = it['values'][0]
        titulo = it['values'][1]
        clasificacion = it['values'][2]

        indexLista = list(map(lambda f: [f.getPath(),f.getTitulo(),f.getClasificacion()],self.listadocumentos)).index([path,titulo,clasificacion])

        file = self.listadocumentos[indexLista]

        editor = TextoDocumento()
        editor.muestraContenidoDocumento(file)

    def __eventoSelItem(self,evento):
        itemSel = evento.widget.item(self.frame_documentos.getTree().focus())
        item = itemSel['values']
        if(len(item)>1):
            tituloSeleccionado = item[1]
            self.titulo_valor.set(tituloSeleccionado)
            if(len(item)>2):
                clasSeleccionado = item[2]
                self.clasificacion_valor.set(clasSeleccionado)

    def __guardarAtributos(self):
        indexItem = self.frame_documentos.getTree().selection()
        path = self.frame_documentos.getTree().item(indexItem)['values'][0]
        titulo = self.frame_documentos.getTree().item(indexItem)['values'][1]
        clasificacion = self.frame_documentos.getTree().item(indexItem)['values'][2]

        self.frame_documentos.getTree().set(indexItem,column='Título',value=self.titulo_valor.get())
        self.frame_documentos.getTree().set(indexItem,column='Clasificación',value=self.clasificacion_valor.get())

        indexLista = list(map(lambda f: [f.getPath(),f.getTitulo(),f.getClasificacion()],self.listadocumentos)).index([path,titulo,clasificacion])

        self.listadocumentos[indexLista].setTitulo(self.titulo_valor.get())
        self.listadocumentos[indexLista].setClasificacion(self.clasificacion_valor.get())

from tabulate import tabulate

from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg)


class Entrena(tk.Toplevel):
    """
    Vista del caso de uso Crear Experimento Entrenamiento. Muestra al usuario la lista de documentos en el sistema, opciones para la validación, le permite definir los parámetros de las redes, y muestra el progreso y resultado del entrenamiento.

    Atributos
    ----------------------
    controlador (controlador.ControladorEntrena): Instancia del controlador que controla esta vista.
    """
    def __init__(self,controlador):
        tk.Toplevel.__init__(self)

        self.iconbitmap('icono.ico')
        self.controlador = controlador

        self.title("Experimento de entrenamiento")

        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(0,weight=1)

    def muestraErrorSinDocumentos(self):
        """
        Muestra un mensaje de error indicando que no existen documentos en el sistema.
        """
        tk.messagebox.showerror("No hay documentos","No se han detectado documentos con clasificación en el sistema. Por favor, importe al menos un documento mediante la opción de Importar Fichero.")

    def cargaPantallaSeleccion(self,documentos):
        """
        Carga la pantalla de selección con los documentos dados, las opciones para la validación y los parámetros de las redes.

        Argumentos
        -------------------
        documentos (list: modelo.Documento): Lista de documentos a mostrar.
        """
        for w in self.winfo_children():
            w.destroy()#limpiamos la pantalla anterior

        self.pantalla = tk.Frame(self)

        self.pantalla.grid_columnconfigure(0,weight=1)
        self.pantalla.grid_columnconfigure(1,weight=1)
        self.pantalla.grid_columnconfigure(2,weight=1)
        self.pantalla.grid_rowconfigure(0,weight=1)

      #Documentos

        # Frame seleccion para entrenamiento
        self.training = ttk.LabelFrame(self.pantalla,text="Documentos a utilizar")
        self.training.grid(row=0,column=0,columnspan=3,sticky='nsew',padx=20,pady=10)
        self.training.grid_columnconfigure(0,weight=1)
        self.training.grid_rowconfigure(1,weight=1)

        self.trainingSel = FramesSeleccion(self.training)
        self.trainingSel.grid(row=1,column=0,sticky='nsew',padx=20,pady=10)

        # Frame seleccion del método de validación
        validacion = ttk.LabelFrame(self.pantalla,text = "Método de validación")
        validacion.grid(row=1,column=0,sticky='nswe',padx=5,pady=10)

        validacion.grid_columnconfigure(1,weight=1)
        validacion.grid_columnconfigure(0,weight=2)

        self.tipoValidacionValue = tk.StringVar()
        self.parametroHoldoutValue = tk.StringVar()
        self.parametroHoldoutValue.set("0,3")
        self.parametroCruzadaValue = tk.IntVar()
        self.parametroCruzadaValue.set(10)

        default = ttk.Radiobutton(validacion,text="Usar conjunto de entrenamiento",variable=self.tipoValidacionValue,value="entrenamiento",command=self.__desactivarResto)
        default.grid(row=0,column=0,sticky='w',pady=10,padx=20)

        ttk.Radiobutton(validacion,text="Holdout",variable=self.tipoValidacionValue,value="holdout",command=self.__activarHoldout).grid(row=1,column=0,sticky='w',padx=20, pady=10)
        ttk.Radiobutton(validacion,text="Validación Cruzada",variable=self.tipoValidacionValue,value="cruzada",command = self.__activarCruzada).grid(row=3,column=0,sticky='w',padx=20,pady=10)

        ttk.Label(validacion,text="Proporción para validación").grid(row=2,column=0,sticky='w',padx=35)
        ttk.Label(validacion,text="Número de subconjuntos").grid(row=4,column=0,sticky='w',padx=35)

        self.inputHoldout = ttk.Entry(validacion,textvariable=self.parametroHoldoutValue,exportselection=0)
        self.inputCruzada = ttk.Entry(validacion,textvariable=self.parametroCruzadaValue,exportselection=0)

        self.inputHoldout.grid(row=2,column=1,sticky='w',padx=5)
        self.inputCruzada.grid(row=4,column=1,sticky='w',padx=5)

        default.invoke()

      #Parámetros
        parametros = ttk.LabelFrame(self.pantalla,text="Parámetros de la red")
        parametros.grid(row=1,column=1, sticky='nswe',padx=5,pady=10)

        parametros.grid_columnconfigure(1,weight=2)
        parametros.grid_columnconfigure(0,weight=1)

        tipoRedLabel = ttk.Label(parametros,text="Tipo de Red:")
        tipoRedLabel.grid(row=0,column=0,sticky='e',padx=5,pady=5)
        self.tipoRedValue = tk.StringVar()
        self.tipoRedInput = ttk.Combobox(parametros, exportselection=0, textvariable=self.tipoRedValue, values=["Elman", "Jordan"])
        self.tipoRedInput.set("Elman")
        self.tipoRedInput.grid(row=0,column=1,sticky='w',pady=5,padx=5,columnspan=2)

        neuronasLabel = ttk.Label(parametros,text="Neuronas en la capa oculta:")
        neuronasLabel.grid(row=1,column=0,sticky='e',padx=5,pady=5)
        self.neuronasValue = tk.StringVar()
        self.neuronasValue.set("20")
        self.neuronasInput = ttk.Entry(parametros, exportselection=0, textvariable=self.neuronasValue)
        self.neuronasInput.grid(row=1,column=1,sticky='w',pady=5,padx=5,columnspan=2)

        factorAprendizajeLabel = ttk.Label(parametros,text="Factor de aprendizaje:")
        factorAprendizajeLabel.grid(row=2,column=0,sticky='e',padx=5,pady=5)
        self.factorAprendizajeValue = tk.StringVar()
        self.factorAprendizajeValue.set("0,8")
        self.factorAprendizajeInput = ttk.Entry(parametros, exportselection=0, textvariable=self.factorAprendizajeValue)
        self.factorAprendizajeInput.grid(row=2,column=1,sticky='w',pady=5,padx=5,columnspan=2)

        factorInerciaLabel = ttk.Label(parametros,text="Factor de inercia:")
        factorInerciaLabel.grid(row=3,column=0,sticky='e',padx=5,pady=5)
        self.factorInerciaValue = tk.StringVar()
        self.factorInerciaValue.set("0,3")
        self.factorInerciaInput = ttk.Entry(parametros, exportselection=0, textvariable=self.factorInerciaValue)
        self.factorInerciaInput.grid(row=3,column=1,sticky='w',pady=5,padx=5,columnspan=2)

        numeroEpocasLabel = ttk.Label(parametros,text="Número de épocas:")
        numeroEpocasLabel.grid(row=4,column=0,sticky='e',padx=5,pady=5)
        self.numeroEpocasValue = tk.StringVar()
        self.numeroEpocasValue.set("200")
        self.numeroEpocasInput = ttk.Entry(parametros, exportselection=0, textvariable=self.numeroEpocasValue)
        self.numeroEpocasInput.grid(row=4,column=1,sticky='w',pady=5,padx=5,columnspan=2)

        #Frame selección de semilla aleatoria, stopwords extra, num documentos necesarios, y asignación de vocabulario
        parametrosextra = ttk.LabelFrame(self.pantalla,text="Parámetros del experimento")
        parametrosextra.grid(row=1,column=2,sticky='nsew',padx=5,pady=10)

        parametrosextra.grid_columnconfigure(1,weight=2)
        parametrosextra.grid_columnconfigure(0,weight=1)

        semillaLabel = ttk.Label(parametrosextra,text="Semilla aleatoria:")
        semillaLabel.grid(row=1,column=0,sticky='e',padx=10,pady=5)
        self.semillaInput = ttk.Entry(parametrosextra, exportselection=0)
        self.semillaInput.grid(row=1,column=1,sticky='w',pady=5,padx=10)

        stopwordsLabel = ttk.Label(parametrosextra,text="Palabras a eliminar de los textos (separadas por comas):")
        stopwordsLabel.grid(row=2,column=0,sticky='e',padx=5,pady=5)
        self.stopwordsInput = ttk.Entry(parametrosextra, exportselection=0)
        self.stopwordsInput.grid(row=2,column=1,sticky='w',pady=5,padx=5,columnspan=2)

        tk.Label(parametrosextra,text="Asignación de valores al vocabulario: ").grid(row=3,column=0,sticky='w',pady=5,padx=5)
        self.tipoVocabulario = tk.StringVar()
        ttk.Radiobutton(parametrosextra,text="Por orden de aparición",variable=self.tipoVocabulario, value="aparicion").grid(row=4,column=0,sticky='w',pady=5,padx=35)
        ttk.Radiobutton(parametrosextra,text="Aleatoriamente",variable=self.tipoVocabulario, value="aleatorio").grid(row=5,column=0,sticky='w',pady=5,padx=35)
        ttk.Radiobutton(parametrosextra,text="Por orden alfabético",variable=self.tipoVocabulario, value="lexicografico").grid(row=6,column=0,sticky='w',pady=5,padx=35)
        self.tipoVocabulario.set("aparicion")

        idiomaLabel = tk.Label(parametrosextra,text="Idioma de los textos: ")
        idiomaLabel.grid(row=7,column=0,sticky='e',padx=5,pady=5)
        self.idiomaValue = tk.StringVar()
        idiomaInput = ttk.Combobox(parametrosextra, exportselection=0, textvariable=self.idiomaValue, values=["Español", "English"])
        idiomaInput.set("Español")
        idiomaInput.grid(row=7,column=1,sticky='w',pady=5,padx=5,columnspan=2)

        # Botones
        frameBotones = ttk.Frame(self.pantalla)
        frameBotones.grid(column=0,row=2,columnspan=3,padx=20,pady=10,sticky='e')
        frameBotones.grid_columnconfigure(0,weight=1)
        frameBotones.grid_columnconfigure(1,weight=1)
        frameBotones.grid_columnconfigure(2,weight=1)

        cancelar = ttk.Button(frameBotones ,text="Cancelar", command=self.destroy)
        cancelar.grid(row=0,column=0,padx=10)

        siguiente = ttk.Button(frameBotones ,text="Siguiente", command = self.__validar)
        siguiente.grid(row=0,column=1,padx=10)

        self.trainingSel.setHeadings(["#","Título", "Clasificación"])
        self.trainingSel.getSelTree().column(0,width=50,stretch=False)
        self.trainingSel.getNoSelTree().column(0,width=50,stretch=False)
        for f in documentos:
                self.trainingSel.addDat([f.getId(),f.getTitulo(),f.getClasificacion()])

        self.pantalla.grid(row=0,column=0,sticky='nswe')

    def __activarHoldout(self):
        self.inputCruzada.config(state='disabled')
        self.inputHoldout.config(state='normal')

    def __activarCruzada(self):
        self.inputCruzada.config(state='normal')
        self.inputHoldout.config(state='disabled')

    def __desactivarResto(self):
        self.inputCruzada.config(state='disabled')
        self.inputHoldout.config(state='disabled')

    def __validar(self):
        #Validación

        #semilla aleatoria entero
        if(self.semillaInput.get()!=""):
            try:
                int(self.semillaInput.get())
            except:
                tk.messagebox.showerror("Parámetro incorrecto","La semilla aleatoria debe ser un entero.",parent=self)
                return

        #entrenamiento vacio
        if(len(self.trainingSel.getSelection())==0):
            tk.messagebox.showerror("Conjunto de entrenamiento vacío","Por favor, seleccione al menos un documento para el entrenamiento.",parent=self)
            return
        #si cruzada numero de folds correcta
        elif(self.tipoValidacionValue.get()=='cruzada' and self.parametroCruzadaValue.get() < 0):
            tk.messagebox.showerror("Parámetro incorrecto","El número de subconjuntos para validación debe ser positivo.",parent=self)
            return
        #numero de neuronas en la capa oculta número entero positivo
        try:
            nv = int(self.neuronasValue.get())
            if(nv <= 0):
                tk.messagebox.showerror("Parámetro incorrecto","El número de neuronas en la capa oculta debe ser mayor que 0.",parent=self)
                return
        except:
            tk.messagebox.showerror("Parámetro incorrecto","El número de neuronas en la capa oculta debe ser un número.",parent=self)
            return
        #numero de épocas entero positivo
        try:
            nE = int(self.numeroEpocasValue.get())
            if(nE <= 0):
                tk.messagebox.showerror("Parámetro incorrecto","El número de épocas debe ser mayor que 0.",parent=self)
                return
        except:
            tk.messagebox.showerror("Parámetro incorrecto","El número de neuronas en la capa oculta debe ser un número.",parent=self)
            return
        #si holdout proporcion correcta
        parHoldout = self.parametroHoldoutValue.get().replace(',','.')
        try:
            parHoldout = float(parHoldout)
            if(self.tipoValidacionValue.get()=='holdout' and (parHoldout < 0 or parHoldout >= 1)):
                tk.messagebox.showerror("Parámetro incorrecto","La proporción de documentos para validación debe estar en el rango [0,1).",parent=self)
                return
        except:
                tk.messagebox.showerror("Parámetro incorrecto","La proporción de documentos para validación debe ser un número.",parent=self)
                return
        #factores  de aprendizaje e inercia
        fA = self.factorAprendizajeValue.get().replace(',','.')
        try:
            fA=float(fA)
            if(fA<0):
                tk.messagebox.showerror("Parámetro incorrecto","El factor de aprendizaje debe ser positivo.",parent=self)
                return
        except:
            tk.messagebox.showerror("Parámetro incorrecto","El factor de aprendizaje debe ser un número.",parent=self)
            return
        fI = self.factorInerciaValue.get().replace(',','.')
        try:
            fI=float(fI)
            if(fI<0):
                tk.messagebox.showerror("Parámetro incorrecto","El factor de inercia debe ser positivo.",parent=self)
                return
        except:
            tk.messagebox.showerror("Parámetro incorrecto","El factor de inercia debe ser un número.",parent=self)
            return
        else:
            self.__siguiente()

    def __siguiente(self):
            #Preparamos el modelo
            m = modelo.Modelo()
            m.setTipoRed(self.tipoRedValue.get())
            m.setNumeroNeuronasOculta(int(self.neuronasValue.get()))

            #Preparamos documentos de entrenamiento
            conEntr = list()
            for fa in self.trainingSel.getSelection():
                f = modelo.Documento()
                f.setId(fa[0]) #le enviamos el id al controlador y dependemos de que él acceda a la bd para rellenar el resto
                conEntr.append(f)

            #Creamos un nuevo experimento
            exp = modelo.ExperimentoEntrenamiento()
            exp.setConjuntoEntrenamiento(conEntr)

            exp.setMetodoValidacion(self.tipoValidacionValue.get())
            if(self.tipoValidacionValue.get() == 'holdout'):
                exp.setParametroValidacion(float(self.parametroHoldoutValue.get().replace(',','.')))
            elif(self.tipoValidacionValue.get() == 'cruzada'):
                exp.setParametroValidacion(self.parametroCruzadaValue.get())

            if(self.semillaInput.get()!=""):
                exp.setSemillaAleatoria(int(self.semillaInput.get()))

            if(self.stopwordsInput.get()!=""):
                listaPalabras = self.stopwordsInput.get().replace(' ','').split(',')
                exp.setStopWordsExtra(listaPalabras)

            exp.setFactorAprendizaje(float(self.factorAprendizajeValue.get().replace(',','.')))
            exp.setFactorInercia(float(self.factorInerciaValue.get().replace(',','.')))
            exp.setNumeroEpocas(int(self.numeroEpocasValue.get()))
            exp.setModelo(m)
            exp.setOrdenVocabulario(self.tipoVocabulario.get())
            exp.setIdioma(self.idiomaValue.get())

            #creamos las queues
            self.queueResultado = queue.LifoQueue()
            self.queueProgreso  = queue.Queue()

            hiloControlador = threading.Thread(target=self.controlador.runExperimento,args=(exp,self.queueResultado,self.queueProgreso))
            hiloControlador.daemon = True

            self.barraInicial = PantallaProgreso(self,100,"Procesando los documentos seleccionados",'indeterminate')
            self.barraInicial.start()

            hiloControlador.start()

            self.flagCargadoExperimento = False #flag que controla que los datos del experimento solo se carguen una vez
            self.after(100,self.__compruebaProgresoExperimento)


    def __errorConjuntoValidacion(self,tipoError):
        """
        Lanza un mensaje de error sobre el conjunto de validación elegido.

        Argumentos:
            - tipoError (str): {'holdout', 'cruzada'} según el tipo de validación que se ha intentado
        """
        if(tipoError.upper()=='HOLDOUT'):
            tk.messagebox.showerror("Conjunto seleciconado incorrecto","Se ha intentado realizar holdout con un conjunto de documentos cuyas clases no permitían conjuntos de entrenamiento y validación que mantengan las proporciones. Por favor, seleccione al menos dos documentos para cada clase seleccionada.",parent=self.pantalla)
        elif(tipoError.upper()=='CRUZADA'):
            tk.messagebox.showerror("Conjunto seleciconado incorrecto","Se ha intentado realizar validación cruzada con un conjunto de documentos cuyas clases no permitían conjuntos de entrenamiento y validación que mantengan las proporciones. Por favor, seleccione al menos tantos documentos para cada clase seleccionada como subconjuntos.",parent=self.pantalla)

    def cargaPantallaProgreso(self):
            """
            Carga la pantalla de progreso del experimento. En esta pantalla se muestra el número de instancias de entrenamiento, de validación, las clases posibles, su proporción en el conjunto de entrenamiento, y se sigue el progreso de las épocas con las tasas de acierto obtenidas en cada iteración. Estos valores se leen en otro método.
            """

            for w in self.winfo_children():
                w.destroy() #limpiamos la pantalla anterior

            self.pantallaRes = tk.Frame(self)
            self.pantallaRes.grid(row=0,column=0,sticky='nsew',padx=20,pady=10)
            self.pantallaRes.grid_columnconfigure(1,weight=1)
            self.pantallaRes.grid_rowconfigure(0,weight=1)
            self.pantallaRes.grid_rowconfigure(2,weight=1)

            self.frameDatosExperimento = tk.Frame(self.pantallaRes)
            self.frameDatosExperimento.grid(row=0,column=0,sticky='nsew',rowspan=3)
            self.frameDatosExperimento.grid_rowconfigure(2,weight=1)


            tk.Label(self.frameDatosExperimento,text="Conjunto de entrada: ").grid(row=0,column=0,sticky='w')
            tk.Label(self.frameDatosExperimento,text="instancias.").grid(row=0,column=2,sticky='w')

            self.tamEntrenamiento = tk.IntVar()

            tk.Label(self.frameDatosExperimento,textvariable=self.tamEntrenamiento).grid(row=0,column=1,sticky='w')

            self.tamValidacion = tk.IntVar()

            tk.Label(self.frameDatosExperimento,text="Conjunto de validación: ").grid(row=1,column=0,sticky='w')
            tk.Label(self.frameDatosExperimento,text="instancias.").grid(row=1,column=2,sticky='w')
            tk.Label(self.frameDatosExperimento,textvariable=self.tamValidacion).grid(row=1,column=1,sticky='w')

            self.clasesEntrenamiento = ScrollableLabelFrame(self.frameDatosExperimento)
            self.clasesEntrenamiento.grid(row=2,column=0,sticky='nswe',columnspan=3)
            self.clasesEntrenamiento.config(text="Clases")
            self.clasesEntrenamiento.getTree().config(selectmode='none')
            self.clasesEntrenamiento.setHeadings(["Clase","% en conjunto de entrenamiento"])

            self.frameEntrenamiento = tk.Frame(self.pantallaRes)
            self.frameEntrenamiento.grid(row=1,column=1,sticky='nsew',padx=20)

            tk.Frame(self.pantallaRes).grid(row=0,column=1,sticky='nsew',pady=20) #para centrarlo
            tk.Frame(self.pantallaRes).grid(row=2,column=1,sticky='nsew',pady=20) #en la pantalla

            self.frameEntrenamiento.grid_columnconfigure(0,weight=1)
            self.frameEntrenamiento.grid_rowconfigure(1,weight=1)

            self.epocaValue = tk.IntVar()
            self.maxEpocas = tk.IntVar()

            tk.Label(self.frameEntrenamiento,text="Época ").grid(row=1,column=0)

            tk.Label(self.frameEntrenamiento,textvariable=self.epocaValue).grid(row=1,column=1)
            tk.Label(self.frameEntrenamiento,text=" de ").grid(row=1,column=2)
            tk.Label(self.frameEntrenamiento,textvariable=self.maxEpocas).grid(row=1,column=3)

            self.progresoEntrenamiento = ttk.Progressbar(self.frameEntrenamiento,length='1000px')
            self.progresoEntrenamiento.grid(row=2,column=0,columnspan=4)

            tk.Label(self.frameEntrenamiento,text="Tasa de acierto en dataset de entrenamiento ").grid(row=3,column=0)
            self.tasaEntrenamiento = tk.StringVar()
            tk.Label(self.frameEntrenamiento,textvariable=self.tasaEntrenamiento).grid(row=3,column=1)

            tk.Label(self.frameEntrenamiento,text="Tasa de acierto en dataset de validación ").grid(row=4,column=0)
            self.tasaValidacion = tk.StringVar()
            tk.Label(self.frameEntrenamiento,textvariable=self.tasaValidacion).grid(row=4,column=1)


    def __compruebaProgresoExperimento(self):
        #se leen los valores de la queue
        try:
            progreso = self.queueProgreso.get(0)
            if(progreso.split(' ')[0]=='ERR'):
                self.barraInicial.destroy()
                self.__errorConjuntoValidacion(progreso.split(' ')[1])
                return
            elif(progreso=='NUEVO EXPERIMENTO'):
                self.flagCargadoExperimento = False
                self.after(100,self.__compruebaProgresoExperimento)
            elif(progreso=='FIN'):
                resultado = self.queueResultado.get(0)
                #FIN DEL EXPERIMENTO
                self.cargaFinalExperimento(resultado)
        except queue.Empty:
            try:
                resultadoProcesado = self.queueResultado.get(0)
                if(not(self.flagCargadoExperimento)):
                    self.barraInicial.destroy()
                    self.cargaPantallaProgreso()
                    #leemos los valores del experimento del resultado y los cargamos:
                    self.tamEntrenamiento.set(resultadoProcesado.getTamEntrenamiento())
                    self.tamValidacion.set(resultadoProcesado.getTamValidacion())
                    for c,p in zip(resultadoProcesado.getClases(),resultadoProcesado.getProporcionClases()):
                        self.clasesEntrenamiento.addDat([str(c),('{0:.2f}%'.format(p*100)).replace('.',',')])

                    self.maxEpocas.set(resultadoProcesado.getEpocasARealizar())
                    self.progresoEntrenamiento.config(maximum=resultadoProcesado.getEpocasARealizar())
                    #Si hay mas de un experimento ejecutandose, indicamos tambíen por que experimento se llega
                    if(resultadoProcesado.getExperimentosARealizar()>1):
                        tk.Label(self.frameEntrenamiento,text="Realizando experimento ").grid(row=0,column=0)
                        tk.Label(self.frameEntrenamiento,text=resultadoProcesado.getExperimentosRealizados()+1).grid(row=0,column=1)
                        tk.Label(self.frameEntrenamiento,text=" de ").grid(row=0,column=2)
                        tk.Label(self.frameEntrenamiento,text=resultadoProcesado.getExperimentosARealizar()).grid(row=0,column=3)
                    self.flagCargadoExperimento = True

                self.epocaValue.set(resultadoProcesado.getEpocasRealizadas())
                self.progresoEntrenamiento.config(value=resultadoProcesado.getEpocasRealizadas())
                if(resultadoProcesado.getEpocasRealizadas()!=0):
                    self.tasaEntrenamiento.set('{0:.2f}%'.format(resultadoProcesado.getTasasEntrenamiento()[-1]*100).replace('.',','))
                    self.tasaValidacion.set('{0:.2f}%'.format(resultadoProcesado.getTasasValidacion()[-1]*100).replace('.',','))

                self.after(100,self.__compruebaProgresoExperimento)
            except queue.Empty:
                self.after(100,self.__compruebaProgresoExperimento)


    def cargaFinalExperimento(self,resultado):
        """
        Carga la pantalla de final del experimento. Esta pantalla contiene las tasas de acierto de cada conjunto, la matriz de confusión resultante tras la última iteración de la red, y una gráfica con la evolución de las tasas de acierto en el tiempo.

        Argumentos:
            - resultado (modelo.EstadoEntrenamiento): instancia del modelo conteniendo los resultados del experimento.
        """

        for w in self.winfo_children():
            w.destroy() #limpiamos la pantalla anterior

        self.pantallaRes = tk.Frame(self)
        self.pantallaRes.grid(row=0,column=0,sticky='nsew',padx=20,pady=20)
        self.pantallaRes.grid_columnconfigure(1,weight=1)
        self.pantallaRes.grid_rowconfigure(1,weight=1)
        self.pantallaRes.grid_rowconfigure(2,weight=1)

        tk.Label(self.pantallaRes,text="Resultados del entrenamiento:").grid(row=0,column=0,sticky='w',padx=5,pady=5)

        self.tasas = ttk.Treeview(self.pantallaRes,selectmode='none',show='headings')
        self.tasas.grid(row=1,column=0,sticky='nsew',padx=5,pady=5)

        self.frameMatrizConfusion = tk.Frame(self.pantallaRes)
        self.frameMatrizConfusion.grid(row=1,column=1,sticky='nsew',padx=5,pady=5)

        self.frameMatrizConfusion.grid_columnconfigure(0,weight=1)
        self.frameMatrizConfusion.grid_rowconfigure(0,weight=1)

        self.scrollbarMatrizX = tk.Scrollbar(self.frameMatrizConfusion,orient='horizontal')
        self.scrollbarMatrizY = tk.Scrollbar(self.frameMatrizConfusion)

        self.matrizConfusion = tk.Text(self.frameMatrizConfusion,wrap="none",
                                       yscrollcommand=self.scrollbarMatrizY.set,
                                       xscrollcommand=self.scrollbarMatrizX.set)

        self.matrizConfusion.grid(row=0,column=0,sticky='nsew')

        self.scrollbarMatrizY.config(command=self.matrizConfusion.yview)
        self.scrollbarMatrizX.config(command=self.matrizConfusion.xview)

        self.scrollbarMatrizY.grid(row=0,column=1,sticky='ns')
        self.scrollbarMatrizX.grid(row=1,column=0,sticky='we')

        self.grafica = tk.Canvas(self.pantallaRes)
        self.grafica.grid(row=2,column=0,sticky='nsew',padx=5,pady=10)

        self.frameBotones = tk.Frame(self.pantallaRes)
        self.frameBotones.grid(row=3,column=0,sticky='ew',padx=5,pady=10,columnspan=2)
        self.frameBotones.grid_columnconfigure(0,weight=1)
        self.frameBotones.grid_columnconfigure(1,weight=1)
        self.frameBotones.grid_columnconfigure(2,weight=1)

        tk.Button(self.frameBotones,text="Entrenar más épocas",command=self.__comandoEntrenarMasEpocas).grid(row=0,column=0,padx=10)

        tk.Button(self.frameBotones,text="Guardar modelo",command=self.__comandoGuardarModelo).grid(row=0,column=1,padx=10)
        tk.Button(self.frameBotones,text="Cerrar sin guardar",command=self.__comandoCerrarSinGuardar).grid(row=0,column=2,padx=10)

        #Tasas de acierto
        cabecera = ['Conjunto','Tasa de aciertos']
        self.tasas.config(column=list(cabecera))
        for h in cabecera:
            self.tasas.heading(h,text=h)

        self.tasas.insert("","end",values=['Entrenamiento',"{:.2f}%".format(resultado.getTasasEntrenamiento()[-1]*100).replace('.',',')])
        self.tasas.insert("","end",values=['Validación',"{:.2f}%".format(resultado.getTasasValidacion()[-1]*100).replace('.',',')])

        #Matriz
        self.matrizConfusion.insert("end","Matriz de confusión:\n----------------------\n\n")
        res = []
        for i,(f,m) in enumerate(zip(resultado.getMatrizConfusion()['filas'],resultado.getMatrizConfusion()['matriz'])):
            res.append(["("+str(i+1)+") "+f])
            res[-1].extend(m)
            res[-1].extend(' ')

        headers = ['clase real']
        columnas = list(range(1,len(resultado.getMatrizConfusion()['columnas'])+1))
        columnas = ["("+str(c)+")" for c in columnas]
        headers.extend(columnas)
        headers.extend(['<- clasificada como'])

        str_matriz = tabulate(res,headers=headers)
        self.matrizConfusion.insert("end",str_matriz)
        self.matrizConfusion.config(state='disabled') #evitamos que se pueda editar la matriz

        #Gráfica de evolución en el tiempo de las tasas
        fig = Figure(figsize=(5, 4), dpi=100)

        ax = fig.add_subplot(111)

        ax.autoscale(False,axis='y')
        ax.set_ylim((0.1,1.1))

        ax.plot(resultado.getTasasEntrenamiento(),'-r', label="Tasa acierto conjunto entrenamiento")
        ax.plot(resultado.getTasasValidacion(),'-b', label="Tasa de acierto conjunto validación")

        vals = ax.get_yticks()
        ax.set_yticklabels(['{:3.0f}%'.format(x*100) for x in vals])

        ax.set_ylabel("% de instancias clasificadas correctamente")
        ax.set_xlabel("Época")
        ax.minorticks_on()
        ax.tick_params(axis='x',which='minor',bottom=False)

        fig.legend(loc='upper center', ncol=2)
        ax.grid(linestyle='--', linewidth=0.7)

        canvas = FigureCanvasTkAgg(fig, master=self.pantallaRes)  # A tk.DrawingArea.
        canvas.draw()
        canvas.get_tk_widget().grid(row=2,column=0,sticky='nsew',columnspan=2)

    def __comandoEntrenarMasEpocas(self):
        nIt = simpledialog.askstring("Entrenar más épocas","Se dispone a entrenar el modelo obtenido durante más épocas. \nEn el caso de haber utilizado validación cruzada, se entrenará el modelo con la mejor tasa de validación de los obtenidos.\n\n ¿Entrenar cuántas épocas más?",parent=self.pantallaRes)
        if(nIt is not None):
            try:
                nIt = int(nIt)
                if(nIt<=0):
                    tk.messagebox.showerror("Número de épocas incorrecta","El número de épocas debe ser mayor que cero.",parent=self.pantallaRes)
                else:
                    self.queueProgreso = queue.LifoQueue()
                    self.queueResultado = queue.Queue()

                    self.hiloControlador = threading.Thread(target=self.controlador.reanudaExperimento,args=(nIt,self.queueResultado,self.queueProgreso))
                    self.hiloControlador.daemon = True
                    self.hiloControlador.start()
                    self.cargaPantallaProgreso()
                    self.flagCargadoExperimento = False #flag que controla que los datos del experimento solo se carguen una vez
                    self.after(100,self.__compruebaProgresoExperimento)
            except ValueError:
                tk.messagebox.showerror("Número de épocas incorrecta","El número de épocas debe ser un entero.",parent=self.pantallaRes)


    def __comandoGuardarModelo(self):
        nombre = simpledialog.askstring("Guardar modelo","Se dispone a guardar el modelo obtenido en el sistema. \nEn el caso de haber utilizado validación cruzada, se guardará el modelo con la mejor tasa de validación de los obtenidos.\n\n Introduzca un nombre para el modelo:",parent=self.pantallaRes)
        if(nombre is not None):
            self.controlador.guardaModelo(nombre)
            self.destroy()

    def __comandoCerrarSinGuardar(self):
        res = tk.messagebox.askokcancel("Cerrando sin guardar","No se guardará el modelo, ¿seguro que quiere cerrar?",parent=self.pantallaRes)
        if(res):
            self.destroy()

class Clasifica(tk.Toplevel):
    """
    Vista del caso de uso Crear Experimento Clasificación. Muestra al usuario la lista de documentos y modelos en el sistema, y muestra el progreso y el resultado del experimento.

    Atributos:
        - controlador (controlador.ControladorClasifica): Instancia del controlador que controla esta vista.
    """
    def __init__(self,controlador):
        tk.Toplevel.__init__(self)
        self.iconbitmap('icono.ico')
        self.controlador = controlador
        self.title("Experimento de clasificación")
        self.grid_columnconfigure(0,weight=1)
        self.grid_rowconfigure(0,weight=1)

    def muestraErrorSinDocumentos(self):
        """
        Muestra un mensaje de error indicando que no existen documentos en el sistema.
        """
        tk.messagebox.showerror("No hay documentos","No se han detectado documentos en el sistema. Por favor, importe al menos un documento mediante la opción de Importar Fichero.")

    def muestraErrorSinModelos(self):
        """
        Muestra un mensaje de error indicando que no existen modelos en el sistema.
        """
        tk.messagebox.showerror("No hay modelos","No se han detectado modelos en el sistema. Por favor, cree al menos un modelo mediante la opción de Experimento > Entrenamiento.")

    def cargaPantallaSeleccion(self,documentos,modelos):
        """
        Carga una pantalla de selección con los documentos y modelos en el sistema.

        Argumentos:

            - documentos (list:modelo.Documento): lista de documentos a mostrar.
            - modelos (list:modelo.Modelo): lista de modelos a mostrar.
        """
        for w in self.winfo_children():
            w.destroy() #limpiamos la pantalla

        self.grid_rowconfigure(1,weight=1)

        self.seleccionDocumentos = FramesSeleccion(self)
        self.seleccionDocumentos.grid(row=0,column=0,sticky='nswe',pady=10,padx=10)

        self.seleccionDocumentos.setHeadings(["#","Título", "Clasificación"])
        self.seleccionDocumentos.getSelTree().column(0,width=50,stretch=False)
        self.seleccionDocumentos.getNoSelTree().column(0,width=50,stretch=False)

        for f in documentos:
            self.seleccionDocumentos.addDat([f.getId(),f.getTitulo(),f.getClasificacion()])

        #selector de modelos
        self.seleccionModelos = SelectorModelo(self)

        self.seleccionModelos.grid(row=1,column=0,sticky='nsew',pady=10,padx=10)
        self.seleccionModelos.muestraModelos(modelos)

        #Botones
        self.botones = tk.Frame(self)
        self.botones.grid(row=2,column=0,sticky='e',padx=20,pady=10)

        self.cancelar = ttk.Button(self.botones,text="Cancelar",command=self.__cancela)
        self.cancelar.grid(row=1,column=0,padx=10)

        self.ejecutar = ttk.Button(self.botones,text="Ejecutar",command=self.__ejecuta)
        self.ejecutar.grid(row=1,column=1,padx=10)

    def __cancela(self):
        self.destroy()

    def __ejecuta(self):
        #obtenemos los documentos y mostramos mensaje de error si no se han seleccionado
        selecciondocumentos = self.seleccionDocumentos.getSelection()

        if(len(selecciondocumentos)==0):
            tk.messagebox.showerror("Selección de documentos vacía","Por favor, seleccione al menos un documento para la clasificación.",parent=self)

        else:
            documentos = list()
            for documento in selecciondocumentos:
                f = modelo.Documento()
                f.setId(documento[0])

                #el controlador rellenará el resto
                documentos.append(f)

            m=self.seleccionModelos.getSeleccion()

            exp = modelo.ExperimentoClasificacion()

            exp.setModelo(m)
            exp.setConjuntoClasificacion(documentos)

            #creamos las queues
            self.queueProgreso = queue.LifoQueue()
            self.queueDocumentos = queue.Queue()

            self.hiloControlador = threading.Thread(target=self.controlador.runExperimento,args=(exp,self.queueProgreso,self.queueDocumentos))
            self.hiloControlador.daemon = True


            self.cargaFinalExperimento()
            self.modalBarra = PantallaProgreso(self,100,"Clasificando "+str(len(exp.getConjuntoClasificacion()))+" documentos",'indeterminate')
            self.modalBarra.title("Clasificado...")

            self.modalBarra.start()

            self.hiloControlador.start()

            self.after(100,self.__compruebaProgresoClasificado)

    def __compruebaProgresoClasificado(self):
        try:
            progreso = self.queueProgreso.get(0)
            if(progreso == 'FIN'):
                self.modalBarra.destroy()
                for i in range(self.queueDocumentos.qsize()):
                    f = self.queueDocumentos.get(i)
                    self.tablaResultados.addDat([f.getId(),f.getTitulo(),f.getClasificacion()])

            else:
                self.after(100,self.__compruebaProgresoClasificado)
        except queue.Empty:
            self.after(100,self.__compruebaProgresoClasificado)

    def cargaFinalExperimento(self):
        """
        Carga la pantalla de resultados del experimento, donde se muestra la clasificación obtenida para cada documento, y se permite ver los datos originales de cada documento.

        Este método no carga los datos.
        """
        for w in self.winfo_children():
            w.destroy() #limpiamos la pantalla anterior

        self.grid_rowconfigure(0,weight=1)
        self.grid_rowconfigure(1,weight=0)

        self.tablaResultados = ScrollableLabelFrame(self)
        self.tablaResultados.config(text="Clasificaciones obtenidas:")
        self.tablaResultados.grid(row=0,column=0,sticky='nsew',padx=20,pady=10)

        self.tablaResultados.setHeadings(["#","Título","Clasificación obtenida"])

        self.tablaResultados.getTree().bind("<Double-ButtonRelease-1>",self.__seleccionaDocumento)

        frameBotones = tk.Frame(self)
        frameBotones.grid(row=1,column=0,padx=20,pady=10,sticky='e')

        aceptar = ttk.Button(frameBotones ,text="Aceptar", command=self.destroy)
        aceptar.grid(row=0,column=0,padx=10,sticky='e')

    def __seleccionaDocumento(self,evento):
        w = evento.widget
        datos_documento = w.item(w.selection())['values']
        file = modelo.Documento()
        file.setId(datos_documento[0])

        self.controlador.cargaContenidoDocumento(file)

    def muestraContenidoDocumento(self,documento):
        """
        Crea una instancia del visor de documentos interno con clase elementosReutilizablesVista.TextoDocumento y carga el contenido del documento dado.

        Argumentos:
            - documento (modelo.Documento): documento a mostrar.
        """
        editor = TextoDocumento()
        editor.muestraContenidoDocumento(documento)
